# General layout:
#
# trio/_core/... is the self-contained core library. It does various
# shenanigans to export a consistent "core API", but parts of the core API are
# too low-level to be recommended for regular use. These are marked by having
# a _hazmat=True attribute.
#
# trio/*.py define a set of more usable tools on top of this. They import from
# trio._core and from each other.
#
# This file pulls together the friendly public API, by re-exporting the more
# innocuous bits of the _core API + the the tools from trio/*.py. No-one
# imports it internally; it's only for public consumption. When re-exporting
# _core here, we check for the _hazmat=True attribute and shunt things into
# either our namespace or the hazmat namespace accordingly.

__all__ = []

from ._version import __version__

from . import hazmat

from . import _core
for _symbol in _core.__all__:
    _value = getattr(_core, _symbol)
    if getattr(_value, "_hazmat", False):
        setattr(hazmat, _symbol, _value)
        hazmat.__all__.append(_symbol)
    else:
        globals()[_symbol] = _value
        __all__.append(_symbol)
del _symbol, _value

from ._timeouts import *
__all__ += _timeouts.__all__

from ._sync import *
__all__ += _sync.__all__

from ._threads import *
__all__ += _threads.__all__

from ._streams import *
__all__ += _streams.__all__

from ._signals import *
__all__ += _signals.__all__

# Imported by default
from . import socket
from . import abc
# Not imported by default: testing



import pytest
import time

from .. import _core
from ..testing import assert_yields
from .._timeouts import *

async def check_takes_about(f, expected_dur):
    start = time.monotonic()
    result = await _core.Result.acapture(f)
    dur = time.monotonic() - start
    # In practice on my Linux laptop I get numbers like 1.003, but Windows is
    # really sloppy -- I've seen 1.6x or so fairly often.
    print(dur / expected_dur)
    assert 1 <= (dur / expected_dur) < 2
    return result.unwrap()

# How long to (attempt to) sleep for when testing. Smaller numbers make the
# test suite go faster.
TARGET = 0.05

async def test_sleep():
    async def sleep_1():
        await sleep_until(_core.current_time() + TARGET)
    await check_takes_about(sleep_1, TARGET)

    async def sleep_2():
        await sleep(TARGET)
    await check_takes_about(sleep_2, TARGET)

    with pytest.raises(ValueError):
        await sleep(-1)

    with assert_yields():
        await sleep(0)
    # This also serves as a test of the trivial move_on_at
    with move_on_at(_core.current_time()):
        with pytest.raises(_core.Cancelled):
            await sleep(0)


async def test_move_on_after():
    with pytest.raises(ValueError):
        with move_on_after(-1):
            pass  # pragma: no cover

    async def sleep_3():
        with move_on_after(TARGET):
            await sleep(100)
    await check_takes_about(sleep_3, TARGET)


async def test_fail():
    async def sleep_4():
        with fail_at(_core.current_time() + TARGET):
            await sleep(100)
    with pytest.raises(TooSlowError):
        await check_takes_about(sleep_4, TARGET)

    with fail_at(_core.current_time() + 100):
        await sleep(0)

    async def sleep_5():
        with fail_after(TARGET):
            await sleep(100)
    with pytest.raises(TooSlowError):
        await check_takes_about(sleep_5, TARGET)

    with fail_after(100):
        await sleep(0)

    with pytest.raises(ValueError):
        with fail_after(-1):
            pass  # pragma: no cover



import threading
import queue as stdlib_queue
import time
import os
import signal

import pytest

from .. import _core
from .. import Event
from ..testing import wait_all_tasks_blocked
from .._threads import *
from .._timeouts import sleep

from .._core.tests.test_ki import ki_self

async def test_do_in_trio_thread():
    trio_thread = threading.current_thread()

    async def check_case(do_in_trio_thread, fn, expected):
        record = []
        def threadfn():
            try:
                record.append(("start", threading.current_thread()))
                x = do_in_trio_thread(fn, record)
                record.append(("got", x))
            except BaseException as exc:
                print(exc)
                record.append(("error", type(exc)))
        child_thread = threading.Thread(target=threadfn, daemon=True)
        child_thread.start()
        while child_thread.is_alive():
            print("yawn")
            await sleep(0.01)
        assert record == [
            ("start", child_thread), ("f", trio_thread), expected]

    run_in_trio_thread = current_run_in_trio_thread()

    def f(record):
        assert not _core.currently_ki_protected()
        record.append(("f", threading.current_thread()))
        return 2
    await check_case(run_in_trio_thread, f, ("got", 2))

    def f(record):
        assert not _core.currently_ki_protected()
        record.append(("f", threading.current_thread()))
        raise ValueError
    await check_case(run_in_trio_thread, f, ("error", ValueError))

    await_in_trio_thread = current_await_in_trio_thread()

    async def f(record):
        assert not _core.currently_ki_protected()
        await _core.yield_briefly()
        record.append(("f", threading.current_thread()))
        return 3
    await check_case(await_in_trio_thread, f, ("got", 3))

    async def f(record):
        assert not _core.currently_ki_protected()
        await _core.yield_briefly()
        record.append(("f", threading.current_thread()))
        raise KeyError
    await check_case(await_in_trio_thread, f, ("error", KeyError))


async def test_do_in_trio_thread_from_trio_thread():
    run_in_trio_thread = current_run_in_trio_thread()
    await_in_trio_thread = current_await_in_trio_thread()

    with pytest.raises(RuntimeError):
        run_in_trio_thread(lambda: None)  # pragma: no branch

    async def foo():  # pragma: no cover
        pass
    with pytest.raises(RuntimeError):
        await_in_trio_thread(foo)


def test_run_in_trio_thread_ki():
    # if we get a control-C during a run_in_trio_thread, then it propagates
    # back to the caller (slick!)
    record = set()
    async def check_run_in_trio_thread():
        run_in_trio_thread = current_run_in_trio_thread()
        await_in_trio_thread = current_await_in_trio_thread()
        def trio_thread_fn():
            print("in trio thread")
            assert not _core.currently_ki_protected()
            print("ki_self")
            try:
                ki_self()
            finally:
                import sys
                print("finally", sys.exc_info())
        async def trio_thread_afn():
            trio_thread_fn()
        def external_thread_fn():
            try:
                print("running")
                run_in_trio_thread(trio_thread_fn)
            except KeyboardInterrupt:
                print("ok1")
                record.add("ok1")
            try:
                await_in_trio_thread(trio_thread_afn)
            except KeyboardInterrupt:
                print("ok2")
                record.add("ok2")
        thread = threading.Thread(target=external_thread_fn)
        thread.start()
        print("waiting")
        while thread.is_alive():
            await sleep(0.01)
        print("waited, joining")
        thread.join()
        print("done")
    _core.run(check_run_in_trio_thread)
    assert record == {"ok1", "ok2"}


def test_await_in_trio_thread_while_main_exits():
    record = []
    ev = Event()

    async def trio_fn():
        record.append("sleeping")
        ev.set()
        await _core.yield_indefinitely(lambda _: _core.Abort.SUCCEEDED)

    def thread_fn(await_in_trio_thread):
        try:
            await_in_trio_thread(trio_fn)
        except _core.Cancelled:
            record.append("cancelled")

    async def main():
        aitt = current_await_in_trio_thread()
        thread = threading.Thread(target=thread_fn, args=(aitt,))
        thread.start()
        await ev.wait()
        assert record == ["sleeping"]
        return thread

    thread = _core.run(main)
    thread.join()
    assert record == ["sleeping", "cancelled"]


async def test_run_in_worker_thread():
    trio_thread = threading.current_thread()

    def f(x):
        return (x, threading.current_thread())
    x, child_thread = await run_in_worker_thread(f, 1)
    assert x == 1
    assert child_thread != trio_thread

    def g():
        raise ValueError(threading.current_thread())
    with pytest.raises(ValueError) as excinfo:
        await run_in_worker_thread(g)
    print(excinfo.value.args)
    assert excinfo.value.args[0] != trio_thread


async def test_run_in_worker_thread_cancellation():
    register = [None]
    def f(q):
        # Make the thread block for a controlled amount of time
        register[0] = "blocking"
        q.get()
        register[0] = "finished"

    async def child(q, cancellable):
        return await run_in_worker_thread(f, q, cancellable=cancellable)

    q = stdlib_queue.Queue()
    async with _core.open_nursery() as nursery:
        task1 = nursery.spawn(child, q, True)
        # Give it a chance to get started. (This is important because
        # run_in_worker_thread does a yield_if_cancelled before blocking on
        # the thread, and we don't want to trigger this.)
        await wait_all_tasks_blocked()
        # Then cancel it.
        nursery.cancel_scope.cancel()
    # The task exited, but the thread didn't:
    assert register[0] != "finished"
    # Put the thread out of its misery:
    q.put(None)
    while register[0] != "finished":
        time.sleep(0.01)

    # This one can't be cancelled
    register[0] = None
    async with _core.open_nursery() as nursery:
        task2 = nursery.spawn(child, q, False)
        await wait_all_tasks_blocked()
        nursery.cancel_scope.cancel()
        with _core.open_cancel_scope(shield=True):
            for _ in range(10):
                await _core.yield_briefly()
        # It's still running
        assert task2.result is None
        q.put(None)
        # Now it exits

    # But if we cancel *before* it enters, the entry is itself a cancellation
    # point
    with _core.open_cancel_scope() as scope:
        scope.cancel()
        await child(q, False)
    assert scope.cancelled_caught


# Make sure that if trio.run exits, and then the thread finishes, then that's
# handled gracefully. (Requires that the thread result machinery be prepared
# for call_soon to raise RunFinishedError.)
def test_run_in_worker_thread_abandoned(capfd):
    q1 = stdlib_queue.Queue()
    q2 = stdlib_queue.Queue()

    def thread_fn():
        q1.get()
        q2.put(threading.current_thread())

    async def main():
        async def child():
            await run_in_worker_thread(thread_fn, cancellable=True)
        async with _core.open_nursery() as nursery:
            t = nursery.spawn(child)
            await wait_all_tasks_blocked()
            nursery.cancel_scope.cancel()
    _core.run(main)

    q1.put(None)
    # This makes sure:
    # - the thread actually ran
    # - that thread has finished before we check for its output
    thread = q2.get()
    while thread.is_alive():
        time.sleep(0.01)  # pragma: no cover

    # Make sure we don't have a "Exception in thread ..." dump to the console:
    out, err = capfd.readouterr()
    assert not out and not err





# A sphinx extension to help documenting Python code that uses async/await (or
# context managers, or abstract methods, or ...).

# Inspirations:
#   https://github.com/python/cpython/blob/master/Doc/tools/extensions/pyspecific.py
#   https://github.com/dabeaz/curio/blob/master/docs/customization.py
#   https://github.com/aio-libs/sphinxcontrib-asyncio/blob/master/sphinxcontrib/asyncio.py

# We take a somewhat different approach, though, based on the observation that
# function properties like "classmethod", "async", "abstractmethod" can be
# mixed and matched, so the the classic sphinx approach of defining different
# directives for all of these quickly becomes cumbersome. Instead, we override
# the ordinary function & method directives to add options corresponding to
# these different properties, and override the autofunction and automethod
# directives to sniff for these properties. Examples:
#
# A function that returns a context manager:
#
#    .. function:: foo(x, y)
#       :with: bar
#
# renders in the docs like:
#
#    with foo(x, y) as bar
#
# The 'bar' part is optional. Use :async-with: for an async context
# manager. These are also accepted on method, autofunction, and automethod.
#
# An abstract async classmethod:
#
#    .. method:: foo
#       :abstractmethod:
#       :classmethod:
#       :async:
#
# renders like:
#
#    abstractmethod classmethod await foo()
#
# Or since all of these attributes are introspectable, we can get the same
# result with:
#
#    .. automethod:: foo
#
# An abstract static decorator:
#
#    .. method:: foo
#       :abstractmethod:
#       :staticmethod:
#       :decorator:
#
# The :decorator: attribute isn't introspectable, but the others
# are, so this also works:
#
#    .. automethod:: foo
#       :decorator:
#
# and renders like
#
#    abstractmethod staticmethod @foo()

# TODO:
#
# - A minor infelicity is that it's possible to explicitly document methods as
#   being a property -- which is useful for cases like
#
#      .. method:: foo
#         :property:
#         :abstractmethod:
#
#   -- but our autodoc classes never generate this (they don't pick up
#   properties at all). Most of the time this is actually reasonable since
#   you usually want to treat properties as attributes rather than methods, or
#   even calling out their property-ness at all. Not sure if there's a better
#   way to handle this. Maybe override attribute and autoattribute too, so it
#   can take an :abstractproperty: flag?
#
# - maybe add :for: and :async-for: to match :with: and :async-with:? rendered
#   like
#     for ... in
#   or
#     for ARGVALUE in

import inspect
import async_generator

from docutils.parsers.rst import directives
from sphinx import addnodes
from sphinx.domains.python import PyModulelevel, PyClassmember
from sphinx.ext.autodoc import (
    FunctionDocumenter, MethodDocumenter, ClassLevelDocumenter,
)

extended_function_option_spec = {
    "async": directives.flag,
    "decorator": directives.flag,
    "with": directives.unchanged,
    "async-with": directives.unchanged,
}

extended_method_option_spec = {
    **extended_function_option_spec,
    "abstractmethod": directives.flag,
    "staticmethod": directives.flag,
    "classmethod": directives.flag,
    "property": directives.flag,
}


################################################################
# Extending the basic function and method directives
################################################################

class ExtendedCallableMixin:
    def needs_arglist(self):
        if "property" in self.options:
            return False
        if "decorator" in self.options or self.objtype == "decorator":
            return False
        return True

    def get_signature_prefix(self, sig):
        ret = ""
        if "abstractmethod" in self.options:
            ret += "abstractmethod "
        # objtype checks are for backwards compatibility, to support
        #
        #   .. staticmethod::
        #
        # in addition to
        #
        #   .. method::
        #      :staticmethod:
        #
        # it would be nice if there were a central place we could normalize
        # the directive name into the options dict instead of having to check
        # both here at time-of-use, but I don't understand sphinx well enough
        # to do that.
        if "staticmethod" in self.options or self.objtype == "staticmethod":
            ret += "staticmethod "
        if "classmethod" in self.options or self.objtype == "classmethod":
            ret += "classmethod "
        if "property" in self.options:
            ret += "property "
        if "with" in self.options:
            ret += "with "
        if "async-with" in self.options:
            ret += "async with "
        if "async" in self.options:
            ret += "await "
        return ret

    def handle_signature(self, sig, signode):
        ret = super().handle_signature(sig, signode)
        if "decorator" in self.options or self.objtype == "decorator":
            signode.insert(0, addnodes.desc_addname("@", "@"))
        for optname in ["with", "async-with"]:
            if self.options.get(optname, "").strip():
                # for some reason a regular space here gets stripped, so we
                # use U+00A0 NO-BREAK SPACE
                s = "\u00A0as {}".format(self.options[optname])
                signode += addnodes.desc_annotation(s, s)
        return ret

class ExtendedPyFunction(ExtendedCallableMixin, PyModulelevel):
    option_spec = {
        **PyModulelevel.option_spec,
        **extended_function_option_spec,
    }

class ExtendedPyMethod(ExtendedCallableMixin, PyClassmember):
    option_spec = {
        **PyClassmember.option_spec,
        **extended_method_option_spec,
    }


################################################################
# Autodoc
################################################################

def sniff_options(obj):
    options = set()
    async_gen = False
    # We walk the __wrapped__ chain to collect properties.
    #
    # If something sniffs as *both* an async generator *and* a coroutine, then
    # it's probably an async_generator-style async_generator (since they wrap
    # a coroutine, but are not a coroutine).
    while True:
        if getattr(obj, "__isabstractmethod__", False):
            options.add("abstractmethod")
        if isinstance(obj, classmethod):
            options.add("classmethod")
        if isinstance(obj, staticmethod):
            options.add("staticmethod")
        if isinstance(obj, property):
            options.add("property")
        if inspect.iscoroutinefunction(obj):
            options.add("async")
        if async_generator.isasyncgenfunction(obj):
            async_gen = True
        if hasattr(obj, "__wrapped__"):
            obj = obj.__wrapped__
        else:
            break
    if async_gen:
        options.discard("async")
    return options

def update_with_sniffed_options(obj, option_dict):
    sniffed = sniff_options(obj)
    for attr in sniffed:
        option_dict[attr] = None

def passthrough_option_lines(self, option_spec):
    sourcename = self.get_sourcename()
    for option in option_spec:
        if option in self.options:
            if self.options.get(option) is not None:
                line = "   :{}: {}".format(option, self.options[option])
            else:
                line = "   :{}:".format(option)
            self.add_line(line, sourcename)

class ExtendedFunctionDocumenter(FunctionDocumenter):
    priority = FunctionDocumenter.priority + 1
    # You can explicitly set the options in case autodetection fails
    option_spec = {
        **FunctionDocumenter.option_spec,
        **extended_function_option_spec,
    }

    def add_directive_header(self, sig):
        super().add_directive_header(sig)
        passthrough_option_lines(self, extended_function_option_spec)

    def import_object(self):
        ret = super().import_object()
        update_with_sniffed_options(self.object, self.options)
        return ret

class ExtendedMethodDocumenter(MethodDocumenter):
    priority = MethodDocumenter.priority + 1
    # You can explicitly set the options in case autodetection fails
    option_spec = {
        **MethodDocumenter.option_spec,
        **extended_method_option_spec,
    }

    def add_directive_header(self, sig):
        super().add_directive_header(sig)
        passthrough_option_lines(self, extended_method_option_spec)

    def import_object(self):
        # MethodDocumenter overrides import_object to do some sniffing in
        # addition to just importing. But we do our own sniffing and just want
        # the import, so we un-override it.
        ret = ClassLevelDocumenter.import_object(self)
        # If you have a classmethod or staticmethod, then
        #
        #   Class.__dict__["name"]
        #
        # returns the classmethod/staticmethod object, but
        #
        #   getattr(Class, "name")
        #
        # returns a regular function. We want to detect
        # classmethod/staticmethod, so we need to go through __dict__.
        obj = self.parent.__dict__.get(self.object_name)
        update_with_sniffed_options(obj, self.options)
        # Replicate the special ordering hacks in
        # MethodDocumenter.import_object
        if "classmethod" in self.options or "staticmethod" in self.options:
            self.member_order -= 1
        return ret

################################################################
# Register everything
################################################################

def setup(app):
    app.add_directive_to_domain('py', 'function', ExtendedPyFunction)
    app.add_directive_to_domain('py', 'method', ExtendedPyMethod)
    app.add_directive_to_domain('py', 'classmethod', ExtendedPyMethod)
    app.add_directive_to_domain('py', 'staticmethod', ExtendedPyMethod)
    app.add_directive_to_domain('py', 'decorator', ExtendedPyFunction)
    app.add_directive_to_domain('py', 'decoratormethod', ExtendedPyMethod)

    # We're overriding these on purpose, so disable the warning about it
    del directives._directives["autofunction"]
    del directives._directives["automethod"]
    app.add_autodocumenter(ExtendedFunctionDocumenter)
    app.add_autodocumenter(ExtendedMethodDocumenter)
    return {'version': '0.1', 'parallel_read_safe': True}



import threading
import queue as stdlib_queue
from itertools import count

from . import _core

__all__ = [
    "current_await_in_trio_thread", "current_run_in_trio_thread",
    "run_in_worker_thread",
]

def _await_in_trio_thread_cb(q, afn, args):
    async def await_in_trio_thread_task():
        nonlocal afn
        afn = _core.disable_ki_protection(afn)
        q.put_nowait(await _core.Result.acapture(afn, *args))
    _core.spawn_system_task(await_in_trio_thread_task, name=afn)

def _run_in_trio_thread_cb(q, fn, args):
    fn = _core.disable_ki_protection(fn)
    q.put_nowait(_core.Result.capture(fn, *args))

def _current_do_in_trio_thread(name, cb):
    call_soon = _core.current_call_soon_thread_and_signal_safe()
    trio_thread = threading.current_thread()
    def do_in_trio_thread(fn, *args):
        if threading.current_thread() == trio_thread:
            raise RuntimeError("must be called from a thread")
        q = stdlib_queue.Queue()
        call_soon(cb, q, fn, args)
        return q.get().unwrap()
    do_in_trio_thread.__name__ = name
    return do_in_trio_thread

def current_run_in_trio_thread():
    return _current_do_in_trio_thread(
        "run_in_trio_thread", _run_in_trio_thread_cb)

def current_await_in_trio_thread():
    return _current_do_in_trio_thread(
        "await_in_trio_thread", _await_in_trio_thread_cb)

################################################################

# XX at some point it probably makes sense to implement some sort of thread
# pool? Or at least that's what everyone says.
#
# There are two arguments for thread pools:
# - speed (re-using threads instead of starting new ones)
# - throttling (if you have 1000 tasks, queue them up instead of spawning 1000
#   threads and running out of memory)
#
# Regarding speed, it's not clear how much of an advantage this is. Some
# numbers on my Linux laptop:
#
# Spawning and then joining a thread:
#
# In [25]: %timeit t = threading.Thread(target=lambda: None); t.start(); t.join()
# 10000 loops, best of 3: 44 µs per loop
#
# Using a thread pool:
#
# In [26]: tpp = concurrent.futures.ThreadPoolExecutor()
# In [27]: %timeit tpp.submit(lambda: None).result()
# <warm up run elided>
# In [28]: %timeit tpp.submit(lambda: None).result()
# 10000 loops, best of 3: 40.8 µs per loop
#
# What's a fast getaddrinfo look like?
#
# # with hot DNS cache:
# In [23]: %timeit socket.getaddrinfo("google.com", "80")
# 10 loops, best of 3: 50.9 ms per loop
#
# In [29]: %timeit socket.getaddrinfo("127.0.0.1", "80")
# 100000 loops, best of 3: 9.73 µs per loop
#
#
# So... maybe we can beat concurrent.futures with a super-efficient thread
# pool or something, but there really is not a lot of headroom here.
#
# Of course other systems might be different... here's CPython 3.6 in a
# Virtualbox VM running Windows 10 on that same Linux laptop:
#
# In [13]: %timeit t = threading.Thread(target=lambda: None); t.start(); t.join()
# 10000 loops, best of 3: 127 µs per loop
#
# In [18]: %timeit tpp.submit(lambda: None).result()
# 10000 loops, best of 3: 31.9 µs per loop
#
# So on Windows there *might* be an advantage? You've gotta be doing a lot of
# connections, with very fast DNS indeed, for that 100 us to matter. But maybe
# someone is.
#
#
# Regarding throttling: this is very much a trade-off. On the one hand, you
# don't want to overwhelm the machine, obviously. On the other hand, queueing
# up work on a central thread-pool creates a central coordination point which
# can potentially create deadlocks and all kinds of fun things. This is very
# context dependent. For getaddrinfo, whatever, they'll make progress and
# complete (we hope), and you want to throttle them to some reasonable
# amount. For calling waitpid() (because just say no to SIGCHLD), then you
# really want one thread-per-waitpid(), because for all you know the user has
# written some ridiculous thing like:
#
#   for p in processes:
#       await spawn(p.wait)
#   # Deadlock here if there are enough processes:
#   await some_other_subprocess.wait()
#   for p in processes:
#       p.terminate()
#
# This goes doubly for the sort of wacky thread usage we see in curio.abide
# (though, I'm not sure if that's actually useful in practice in our context,
# run_in_trio_thread seems like it might be a nicer synchronization primitive
# for most uses than trying to make threading.Lock awaitable).
#
# See also this very relevant discussion:
#
#   https://twistedmatrix.com/trac/ticket/5298
#
# "Interacting with the products at Rackspace which use Twisted, I've seen
# problems caused by thread-pool maximum sizes with some annoying
# regularity. The basic problem is this: if you have a hard limit on the
# number of threads, *it is not possible to write a correct program which may
# require starting a new thread to un-block a blocked pool thread*" - glyph
#
# For now, if we want to throttle getaddrinfo I think the simplest thing is
# for the socket code to have a semaphore for getaddrinfo calls.
#
# Regarding the memory overhead of threads, in theory one should be able to
# reduce this a *lot* for a thread that's just calling getaddrinfo or
# (especially) waitpid. Windows and pthreads both offer the ability to set
# thread stack size on a thread-by-thread basis. Unfortunately as of 3.6
# CPython doesn't expose this in a useful way (all you can do is set it
# globally for the whole process, so it's - ironically - not thread safe).
#
# (It's also unclear how much stack size actually matters; on a 64-bit Linux
# server with overcommit -- i.e., the most common configuration -- then AFAICT
# really the only real limit is on stack size actually *used*; how much you
# *allocate* should be pretty much irrelevant.)

_worker_thread_counter = count()

@_core.enable_ki_protection
async def run_in_worker_thread(sync_fn, *args, cancellable=False):
    """Convert a blocking operation in an async operation using a thread.

    These two lines are equivalent::

        sync_fn(*args)
        await run_in_worker_thread(sync_fn, *args)

    except that if ``sync_fn`` takes a long time, then the first line will
    block the Trio loop while it runs, while the second line allows other Trio
    tasks to continue working while ``sync_fn`` runs. This is accomplished by
    pushing the call to ``sync_fn(*args)`` off into a worker thread.

    **Cancellation handling**: Cancellation is a tricky issue here, because
    neither Python nor the operating systems it runs on provide any general
    way to communicate with an arbitrary synchronous function running in a
    thread and tell it to stop. This function will always check for
    cancellation on entry, before starting the thread. But once the thread is
    running, there are two ways it can handle being cancelled:

    * If ``cancellable=False``, the function ignores the cancellation and
      keeps going, just like if we had called ``sync_fn`` synchronously. This
      is the default behavior.

    * If ``cancellable=True``, then ``run_in_worker_thread`` immediately
      raises :exc:`Cancelled`. In this case **the thread keeps running in
      background** – we just abandon it to do whatever it's going to do, and
      silently discard any return value or errors that it raises. Only use
      this if you know that the operation is safe and side-effect free. (For
      example: ``trio.socket.getaddrinfo`` is implemented using
      :func:`run_in_worker_thread`, and it sets ``cancellable=True`` because
      it doesn't really matter if a stray hostname lookup keeps running in the
      background.)

    .. warning::

       You should not use :func:`run_in_worker_thread` to call CPU-bound
       functions! In addition to the usual GIL-related reasons why using
       threads for CPU-bound work is not very effective in Python, there is an
       additional problem: on CPython, `CPU-bound threads tend to "starve out"
       IO-bound threads <https://bugs.python.org/issue7946>`__, so using
       :func:`run_in_worker_thread` for CPU-bound work is likely to adversely
       affect the main thread running trio. If you need to do this, you're
       better off using a worker process, or perhaps PyPy (which still has a
       GIL, but may do a better job of fairly allocating CPU time between
       threads).

    Args:
      sync_fn: An arbitrary synchronous callable.
      *args: Positional arguments to pass to sync_fn. If you need keyword
          arguments, use :func:`functools.partial`.
      cancellable (bool): Whether to allow cancellation of this operation. See
          discussion above.

    Returns:
      Whatever ``sync_fn`` returns.

    Raises:
      Whatever ``sync_fn`` raises.

    """
    await _core.yield_if_cancelled()
    call_soon = _core.current_call_soon_thread_and_signal_safe()
    task_register = [_core.current_task()]
    def trio_thread_fn(result):
        if task_register[0] is not None:
            _core.reschedule(task_register[0], result)
    def worker_thread_fn():
        result = _core.Result.capture(sync_fn, *args)
        try:
            call_soon(trio_thread_fn, result)
        except _core.RunFinishedError:
            # The entire run finished, so our particular task is certainly
            # long gone -- it must have cancelled.
            pass
    name = "trio-worker-{}".format(next(_worker_thread_counter))
    # daemonic because it might get left behind if we cancel
    thread = threading.Thread(target=worker_thread_fn, name=name, daemon=True)
    thread.start()
    def abort(_):
        if cancellable:
            task_register[0] = None
            return _core.Abort.SUCCEEDED
        else:
            return _core.Abort.FAILED
    return await _core.yield_indefinitely(abort)



import time
from math import inf

import pytest

from .. import sleep
from .. import _core
from ..testing import *

async def test_wait_all_tasks_blocked():
    record = []
    async def busy_bee():
        for _ in range(10):
            await _core.yield_briefly()
        record.append("busy bee exhausted")

    async def waiting_for_bee_to_leave():
        await wait_all_tasks_blocked()
        record.append("quiet at last!")

    async with _core.open_nursery() as nursery:
        t1 = nursery.spawn(busy_bee)
        t2 = nursery.spawn(waiting_for_bee_to_leave)
        t3 = nursery.spawn(waiting_for_bee_to_leave)

    # check cancellation
    async def cancelled_while_waiting():
        try:
            await wait_all_tasks_blocked()
        except _core.Cancelled:
            return "ok"
    async with _core.open_nursery() as nursery:
        t4 = nursery.spawn(cancelled_while_waiting)
        nursery.cancel_scope.cancel()
    assert t4.result.unwrap() == "ok"

async def test_wait_all_tasks_blocked_with_timeouts(mock_clock):
    record = []
    async def timeout_task():
        record.append("tt start")
        await sleep(5)
        record.append("tt finished")
    async with _core.open_nursery() as nursery:
        t = nursery.spawn(timeout_task)
        await wait_all_tasks_blocked()
        assert record == ["tt start"]
        mock_clock.jump(10)
        await wait_all_tasks_blocked()
        assert record == ["tt start", "tt finished"]


async def test_wait_all_tasks_blocked_with_cushion():
    record = []
    async def blink():
        record.append("blink start")
        await sleep(0.01)
        await sleep(0.01)
        await sleep(0.01)
        record.append("blink end")

    async def wait_no_cushion():
        await wait_all_tasks_blocked()
        record.append("wait_no_cushion end")

    async def wait_small_cushion():
        await wait_all_tasks_blocked(0.02)
        record.append("wait_small_cushion end")

    async def wait_big_cushion():
        await wait_all_tasks_blocked(0.03)
        record.append("wait_big_cushion end")

    async with _core.open_nursery() as nursery:
        nursery.spawn(blink)
        nursery.spawn(wait_no_cushion)
        nursery.spawn(wait_small_cushion)
        nursery.spawn(wait_small_cushion)
        nursery.spawn(wait_big_cushion)
    assert record == [
        "blink start",
        "wait_no_cushion end",
        "blink end",
        "wait_small_cushion end",
        "wait_small_cushion end",
        "wait_big_cushion end",
    ]

async def test_assert_yields():
    with assert_yields():
        await _core.yield_briefly()

    with pytest.raises(AssertionError):
        with assert_yields():
            1 + 1

    with assert_no_yields():
        1 + 1

    with pytest.raises(AssertionError):
        with assert_no_yields():
            await _core.yield_briefly()

    # partial yield cases
    # if you have a schedule point but not a cancel point, or vice-versa, then
    # that doesn't make *either* version of assert_{no_,}yields happy.
    for partial_yield in [
            _core.yield_if_cancelled, _core.yield_briefly_no_cancel]:
        for block in [assert_yields, assert_no_yields]:
            print(partial_yield, block)
            with pytest.raises(AssertionError):
                with block():
                    await partial_yield()

    # But both together count as a yield point
    with assert_yields():
        await _core.yield_if_cancelled()
        await _core.yield_briefly()
    with pytest.raises(AssertionError):
        with assert_no_yields():
            await _core.yield_if_cancelled()
            await _core.yield_briefly()


async def test_Sequencer():
    record = []
    def t(val):
        print(val)
        record.append(val)

    async def f1(seq):
        async with seq(1):
            t(("f1", 1))
        async with seq(3):
            t(("f1", 3))
        async with seq(4):
            t(("f1", 4))

    async def f2(seq):
        async with seq(0):
            t(("f2", 0))
        async with seq(2):
            t(("f2", 2))

    seq = Sequencer()
    async with _core.open_nursery() as nursery:
        t1 = nursery.spawn(f1, seq)
        t2 = nursery.spawn(f2, seq)
        async with seq(5):
            await t1.wait()
            await t2.wait()
        assert record == [("f2", 0), ("f1", 1), ("f2", 2), ("f1", 3), ("f1", 4)]

    seq = Sequencer()
    # Catches us if we try to re-use a sequence point:
    async with seq(0):
        pass
    with pytest.raises(RuntimeError):
        async with seq(0):
            pass  # pragma: no cover


async def test_Sequencer_cancel():
    # Killing a blocked task makes everything blow up
    record = []
    seq = Sequencer()
    async def child(i):
        with _core.open_cancel_scope() as scope:
            if i == 1:
                scope.cancel()
            try:
                async with seq(i):
                    pass  # pragma: no cover
            except RuntimeError:
                record.append("seq({}) RuntimeError".format(i))

    async with _core.open_nursery() as nursery:
        t1 = nursery.spawn(child, 1)
        t2 = nursery.spawn(child, 2)
        async with seq(0):
            pass  # pragma: no cover

    assert record == ["seq(1) RuntimeError", "seq(2) RuntimeError"]

    # Late arrivals also get errors
    with pytest.raises(RuntimeError):
        async with seq(3):
            pass  # pragma: no cover


def test_mock_clock():
    REAL_NOW = 123.0
    c = MockClock()
    c._real_clock = lambda: REAL_NOW
    repr(c)  # smoke test
    assert c.rate == 0
    assert c.current_time() == 0
    c.jump(1.2)
    assert c.current_time() == 1.2
    with pytest.raises(ValueError):
        c.jump(-1)
    assert c.current_time() == 1.2
    assert c.deadline_to_sleep_time(1.1) == 0
    assert c.deadline_to_sleep_time(1.2) == 0
    assert c.deadline_to_sleep_time(1.3) > 999999

    with pytest.raises(ValueError):
        c.rate = -1
    assert c.rate == 0

    c.rate = 2
    assert c.current_time() == 1.2
    REAL_NOW += 1
    assert c.current_time() == 3.2
    assert c.deadline_to_sleep_time(3.1) == 0
    assert c.deadline_to_sleep_time(3.2) == 0
    assert c.deadline_to_sleep_time(4.2) == 0.5

    c.rate = 0.5
    assert c.current_time() == 3.2
    assert c.deadline_to_sleep_time(3.1) == 0
    assert c.deadline_to_sleep_time(3.2) == 0
    assert c.deadline_to_sleep_time(4.2) == 2.0

    c.jump(0.8)
    assert c.current_time() == 4.0
    REAL_NOW += 1
    assert c.current_time() == 4.5


    c2 = MockClock(rate=3)
    assert c2.rate == 3
    assert c2.current_time() < 10


async def test_mock_clock_autojump(mock_clock):
    assert mock_clock.autojump_threshold == inf

    mock_clock.autojump_threshold = 0
    assert mock_clock.autojump_threshold == 0

    real_start = time.monotonic()

    virtual_start = _core.current_time()
    for i in range(10):
        print("sleeping {} seconds".format(10 * i))
        await sleep(10 * i)
        print("woke up!")
        assert virtual_start + 10 * i == _core.current_time()
        virtual_start = _core.current_time()

    real_duration = time.monotonic() - real_start
    print("Slept {} seconds in {} seconds"
          .format(10 * sum(range(10)), real_duration))
    assert real_duration < 1

    mock_clock.autojump_threshold = 0.02
    t = _core.current_time()
    # this should wake up before the autojump threshold triggers, so time
    # shouldn't change
    await wait_all_tasks_blocked()
    assert t == _core.current_time()
    # this should too
    await wait_all_tasks_blocked(0.01)
    assert t == _core.current_time()

    # This should wake up at the same time as the autojump_threshold, and
    # confuse things. There is no deadline, so it shouldn't actually jump
    # the clock. But does it handle the situation gracefully?
    await wait_all_tasks_blocked(0.02)
    # And again with threshold=0, because that has some special
    # busy-wait-avoidance logic:
    mock_clock.autojump_threshold = 0
    await wait_all_tasks_blocked()

    # set up a situation where the autojump task is blocked for a long long
    # time, to make sure that cancel-and-adjust-threshold logic is working
    mock_clock.autojump_threshold = 10000
    await wait_all_tasks_blocked()
    mock_clock.autojump_threshold = 0
    # if the above line didn't take affect immediately, then this would be
    # bad:
    await sleep(100000)


async def test_mock_clock_autojump_interference(mock_clock):
    mock_clock.autojump_threshold = 0.02

    mock_clock2 = MockClock()
    # messing with the autojump threshold of a clock that isn't actually
    # installed in the run loop shouldn't do anything.
    mock_clock2.autojump_threshold = 0.01

    # if the autojump_threshold of 0.01 were in effect, then the next line
    # would block forever, as the autojump task kept waking up to try to
    # jump the clock.
    await wait_all_tasks_blocked(0.015)

    # but the 0.02 limit does apply
    await sleep(100000)


def test_mock_clock_autojump_preset():
    # Check that we can set the autojump_threshold before the clock is
    # actually in use, and it gets picked up
    mock_clock = MockClock(autojump_threshold=0.1)
    mock_clock.autojump_threshold = 0.01
    real_start = time.monotonic()
    _core.run(sleep, 10000, clock=mock_clock)
    assert time.monotonic() - real_start < 1



import abc
import contextlib

import attr

from . import _core

__all__ = ["AsyncResource", "SendStream", "RecvStream", "Stream"]

# The close API is a big question here.
#
# Technically, socket close can block if you set various weird
# lingering-related options. This doesn't seem very useful though.
#
# For other kinds of channels, though, the natural implementation definitely
# can block – e.g. TLS wants to send a goodbye message, and if we're tunneling
# over ssh or HTTP/2 e.g. then again closing requires sending some actual
# data. So we need a concept of a blocking close.
#
# BUT, if the other side is uncooperative, we can't necessarily block in
# close. So if a blocking close is cancelled, we need to do some sort of
# forceful cleanup before raising the exception.
#
# (Probably implementing H2-based streams will be a useful forcing function
# here to figure this out.)

class AsyncResource(metaclass=abc.ABCMeta):
    __slots__ = ()

    @abc.abstractmethod
    def forceful_close(self):
        """Force an immediate close of this resource.

        This will never block, but (depending on the resource in question) it
        might be a "rude" shutdown.
        """
        pass

    async def graceful_close(self):
        """Close this resource, gracefully.

        This may block in order to perform a "graceful" shutdown (for example,
        sending a message alerting the other side of a connection that it is
        about to close). But, if cancelled, then it still *must* close the
        underlying resource.

        Default implementation is to perform a :meth:`forceful_close` and then
        execute a yield point.
        """
        self.forceful_close()
        await _core.yield_briefly()

    async def __aenter__(self):
        return self

    async def __aexit__(self, *args):
        await self.graceful_close()

# XX added in 3.6
if hasattr(contextlib, "AbstractContextManager"):
    contextlib.AbstractContextManager.register(AsyncResource)

class SendStream(AsyncResource):
    __slots__ = ()

    @abc.abstractmethod
    async def sendall(self, data):
        pass

    # This is only a hint, because in some cases we don't know (Windows), or
    # we have only a noisy signal (TLS). And in the use cases this is included
    # to account for, returning before it's actually writable is NBD, it just
    # makes them slightly less efficient.
    @abc.abstractmethod
    async def wait_maybe_writable(self):
        pass

    @property
    @abc.abstractmethod
    def can_send_eof(self):
        pass

    @abc.abstractmethod
    def send_eof(self):
        pass

class RecvStream(AsyncResource):
    __slots__ = ()

    @abc.abstractmethod
    async def recv(self, max_bytes):
        pass

class Stream(SendStream, RecvStream):
    __slots__ = ()

    @staticmethod
    def staple(send_stream, recv_stream):
        return StapledStream(send_stream=send_stream, recv_stream=recv_stream)

@attr.s(slots=True, cmp=False, hash=False)
class StapledStream(Stream):
    send_stream = attr.ib()
    recv_stream = attr.ib()

    async def sendall(self, data):
        return await self.send_stream.sendall(data)

    async def wait_maybe_writable(self):
        return await self.send_stream.wait_maybe_writable()

    @property
    def can_send_eof(self):
        return self.send_stream.can_send_eof

    def send_eof(self):
        return self.send_stream.send_eof()

    async def recv(self, max_bytes):
        return self.recv_stream.recv(max_bytes)

    def forceful_close(self):
        try:
            self.send_stream.forceful_close()
        finally:
            self.recv_stream.forceful_close()

    async def graceful_close(self):
        try:
            await self.send_stream.graceful_close()
        finally:
            await self.recv_stream.graceful_close()



import select
from contextlib import contextmanager
import attr

from .. import _core
from . import _public, _hazmat

@attr.s(frozen=True)
class _KqueueStatistics:
    tasks_waiting = attr.ib()
    monitors = attr.ib()
    backend = attr.ib(default="kqueue")

@attr.s(slots=True, cmp=False, hash=False)
class KqueueIOManager:
    _kqueue = attr.ib(default=attr.Factory(select.kqueue))
    # {(ident, filter): Task or UnboundedQueue}
    _registered = attr.ib(default=attr.Factory(dict))

    def statistics(self):
        tasks_waiting = 0
        monitors = 0
        for receiver in self._registered.values():
            if type(receiver) is _core.Task:
                tasks_waiting += 1
            else:
                monitors += 1
        return _KqueueStatistics(
            tasks_waiting=tasks_waiting,
            monitors=monitors,
        )

    def close(self):
        self._kqueue.close()

    def handle_io(self, timeout):
        # max_events must be > 0 or kqueue gets cranky
        # and we generally want this to be strictly larger than the actual
        # number of events we get, so that we can tell that we've gotten
        # all the events in just 1 call.
        max_events = len(self._registered) + 1
        events = []
        while True:
            batch = self._kqueue.control([], max_events, timeout)
            events += batch
            if len(batch) < max_events:
                break
            else:
                timeout = 0
                # and loop back to the start
        for event in events:
            key = (event.ident, event.filter)
            receiver = self._registered[key]
            if event.flags & select.KQ_EV_ONESHOT:
                del self._registered[key]
            if type(receiver) is _core.Task:
                _core.reschedule(receiver, _core.Value(event))
            else:
                receiver.put_nowait(event)

    # kevent registration is complicated -- e.g. aio submission can
    # implicitly perform a EV_ADD, and EVFILT_PROC with NOTE_TRACK will
    # automatically register filters for child processes. So our lowlevel
    # API is *very* low-level: we expose the kqueue itself for adding
    # events or sticking into AIO submission structs, and split waiting
    # off into separate methods. It's your responsibility to make sure
    # that handle_io never receives an event without a corresponding
    # registration! This may be challenging if you want to be careful
    # about e.g. KeyboardInterrupt. Possibly this API could be improved to
    # be more ergonomic...

    @_public
    @_hazmat
    def current_kqueue(self):
        return self._kqueue

    @_public
    @_hazmat
    @contextmanager
    def monitor_kevent(self, ident, filter):
        key = (ident, filter)
        if key in self._registered:
            raise RuntimeError(
                "attempt to register multiple listeners for same "
                "ident/filter pair")
        q = _core.UnboundedQueue()
        self._registered[key] = q
        try:
            yield q
        finally:
            del self._registered[key]

    @_public
    @_hazmat
    async def wait_kevent(self, ident, filter, abort_func):
        key = (ident, filter)
        if key in self._registered:
            raise RuntimeError(
                "attempt to register multiple listeners for same "
                "ident/filter pair")
        self._registered[key] = _core.current_task()
        def abort(raise_cancel):
            r = abort_func(raise_cancel)
            if r is _core.Abort.SUCCEEDED:
                del self._registered[key]
            return r
        return await _core.yield_indefinitely(abort)

    async def _wait_common(self, fd, filter):
        if not isinstance(fd, int):
            fd = fd.fileno()
        flags = select.KQ_EV_ADD | select.KQ_EV_ONESHOT
        event = select.kevent(fd, filter, flags)
        self._kqueue.control([event], 0)
        def abort(_):
            event = select.kevent(fd, filter, select.KQ_EV_DELETE)
            self._kqueue.control([event], 0)
            return _core.Abort.SUCCEEDED
        await self.wait_kevent(fd, filter, abort)

    @_public
    @_hazmat
    async def wait_readable(self, fd):
        await self._wait_common(fd, select.KQ_FILTER_READ)

    @_public
    @_hazmat
    async def wait_writable(self, fd):
        await self._wait_common(fd, select.KQ_FILTER_WRITE)



import sys
from functools import wraps

import async_generator

__all__ = ["aitercompat", "acontextmanager"]

# Decorator to handle the change to __aiter__ in 3.5.2
def aiter_compat(aiter_impl):
    if sys.version_info < (3, 5, 2):
        @wraps(aiter_impl)
        async def __aiter__(*args, **kwargs):
            return aiter_impl(*args, **kwargs)
        return __aiter__
    else:
        return aiter_impl

# Very much derived from the one in contextlib, by copy/pasting and then
# asyncifying everything.
# So this is a derivative work licensed under the PSF License, which requires
# the following notice:
#
# Copyright © 2001-2017 Python Software Foundation; All Rights Reserved
class _AsyncGeneratorContextManager:
    def __init__(self, func, args, kwds):
        self._agen = func(*args, **kwds).__aiter__()

    async def __aenter__(self):
        if sys.version_info < (3, 5, 2):
            self._agen = await self._agen
        try:
            return await self._agen.asend(None)
        except StopAsyncIteration:
            raise RuntimeError("async generator didn't yield") from None

    async def __aexit__(self, type, value, traceback):
        if type is None:
            try:
                await self._agen.asend(None)
            except StopAsyncIteration:
                return
            else:
                raise RuntimeError("async generator didn't stop")
        else:
            if value is None:
                # Need to force instantiation so we can reliably
                # tell if we get the same exception back
                value = type()
            try:
                await self._agen.athrow(type, value, traceback)
                raise RuntimeError("async generator didn't stop after athrow()")
            except StopAsyncIteration as exc:
                # Suppress StopIteration *unless* it's the same exception that
                # was passed to throw().  This prevents a StopIteration
                # raised inside the "with" statement from being suppressed.
                return (exc is not value)
            except RuntimeError as exc:
                # Don't re-raise the passed in exception. (issue27112)
                if exc is value:
                    return False
                # Likewise, avoid suppressing if a StopIteration exception
                # was passed to throw() and later wrapped into a RuntimeError
                # (see PEP 479).
                if exc.__cause__ is value:
                    return False
                raise
            except:
                # only re-raise if it's *not* the exception that was
                # passed to throw(), because __exit__() must not raise
                # an exception unless __exit__() itself failed.  But throw()
                # has to raise the exception to signal propagation, so this
                # fixes the impedance mismatch between the throw() protocol
                # and the __exit__() protocol.
                #
                if sys.exc_info()[1] is not value:
                    raise

def acontextmanager(func):
    """Like @contextmanager, but async."""
    if not async_generator.isasyncgenfunction(func):
        raise TypeError(
            "must be an async generator (native or from async_generator; "
            "if using @async_generator then @acontextmanager must be on top.")
    @wraps(func)
    def helper(*args, **kwds):
        return _AsyncGeneratorContextManager(func, args, kwds)
    return helper



import pytest
import sys
import os
import signal
import threading
import contextlib

from async_generator import async_generator, yield_

from ... import _core
from ...testing import wait_all_tasks_blocked
from ..._util import acontextmanager

if os.name == "nt":
    # I looked at this pretty hard and I'm pretty sure there isn't any way to
    # deliver a real CTRL_C_EVENT to the test process that works reliably on
    # Appveyor etc.
    #
    # You can make a synthetic CTRL_C_EVENT using GenerateConsoleCtrlEvent:
    #
    #    https://msdn.microsoft.com/en-us/library/windows/desktop/ms683155(v=vs.85).aspx
    #
    # However, this isn't directed at a *process*. Your options are:
    #
    # 1) send it to process 0, which means "everyone attached to this
    # console".
    #
    # 2) If a process is a group leader, then you can pass in its PID, and
    # which case it might send it to just the processes that are in that group
    # and also share a console with the caller. Or maybe not -- the page above
    # claims that this doesn't work for CTRL_C_EVENT, only
    # CTRL_BREAK_EVENT. But it's clearly wrong, because it did work for me in
    # some cases?
    #
    # 3) If a process is *not* a group leader and you pass in its PID, then
    # the results are not documented. But empirically it seems like it might
    # act like you passed 0:
    #
    #    https://stackoverflow.com/questions/42180468
    #
    # So our problem is that we want to deliver a CTRL_C_EVENT to the current
    # process, without causing side-effects like freezing Appveyor.
    #
    # There are two strategies that suggest themselves: either run pytest as a
    # new process group, so we're a process group leader, or else run pytest
    # on a new console.
    #
    # There are some annoyances here that I do know how to overcome. If
    # creating a new process group: (a) we need to spawn Python directly,
    # e.g. "python -m pytest". If we use a entry-point script like "pytest",
    # or a launcher like "py", then it doesn't work -- probably because then
    # the launching process ends up as group leader, so the actual test
    # process isn't, and that breaks things as per above. (b)
    # CREATE_NEW_PROCESS_GROUP also sets an internal flag that makes it so the
    # spawned process ignores CTRL_C_EVENT. We have to flip that flag back by
    # hand, using SetConsoleCtrlHandler(NULL, FALSE).
    #
    # If creating a new console: we need to hide the new console window
    # (shell=True in subprocess.Popen), and pass in a stdout/stderr PIPE and
    # pump the output back by hand. (Otherwise the output goes to the new
    # console window, which on Appveyor is obviously useless regardless of
    # whether the new window is hidden.) AFAICT there's no way to directly
    # pass in the original console's output handles as handles for the child
    # process, probably because console handles are weird virtual handle
    # things.
    #
    # The docs claim that if you try to do both at once (CREATE_NEW_CONSOLE |
    # CREATE_NEW_PROCESS_GROUP) then that's equivalent to CREATE_NEW_CONSOLE
    # alone. I haven't tried to check this.
    #
    # There is also some subtlety required when using GenerateConsoleCtrlEvent
    # to make sure that the event is actually delivered at the right time --
    # for the tests we want the Python-level signal handler to run before we
    # exit ki_self, but this is tricky because the CTRL_C_EVENT handler runs
    # in a new thread that isn't synchronized to anything.
    #
    # Anyway. Having done all the work to deal with this issues, both of these
    # approaches work... kind of. They work locally. And I was able to get
    # them to work seemingly-reliably on Appveyor (like, passing 8/8 runs),
    # BUT this required weird tweaking -- in particular, the working runs were
    # like
    #
    #    python -u -m pytest -v -s
    #
    # Yes, it is critically important that we run python with unbuffered stdio
    # (-u). Without this then there are freezes on a regular (but not 100%
    # deterministic) basis. Maybe there is something where you have to
    # actually interact with the console in order to get CTRL_C_EVENT
    # delivered? It's totally baffling to me.
    #
    # So anyway, I just don't trust this. I'd much rather have a test suite
    # that runs 100% reliably and tests 99% of the stuff than a test suite
    # that runs 99% reliably and tests 100% of the stuff. So we fake it.
    def ki_self():
        assert threading.current_thread() == threading.main_thread()
        handler = signal.getsignal(signal.SIGINT)
        handler(signal.SIGINT, sys._getframe())
else:
    # On Unix, kill invokes the C handler synchronously, in this process only,
    # and then os.kill immediately checks for this and runs the Python handler
    # before returning. So... that's easy.
    def ki_self():
        os.kill(os.getpid(), signal.SIGINT)

def test_ki_self():
    with pytest.raises(KeyboardInterrupt):
        ki_self()

async def test_ki_enabled():
    # Regular tasks aren't KI-protected
    assert not _core.currently_ki_protected()

    # Low-level call-soon callbacks are KI-protected
    call_soon = _core.current_call_soon_thread_and_signal_safe()
    record = []
    def check():
        record.append(_core.currently_ki_protected())
    call_soon(check)
    await wait_all_tasks_blocked()
    assert record == [True]

    @_core.enable_ki_protection
    def protected():
        assert _core.currently_ki_protected()
        unprotected()

    @_core.disable_ki_protection
    def unprotected():
        assert not _core.currently_ki_protected()

    protected()

    @_core.enable_ki_protection
    async def aprotected():
        assert _core.currently_ki_protected()
        await aunprotected()

    @_core.disable_ki_protection
    async def aunprotected():
        assert not _core.currently_ki_protected()

    await aprotected()

    # make sure that the decorator here overrides the automatic manipulation
    # that spawn() does:
    async with _core.open_nursery() as nursery:
        nursery.spawn(aprotected)
        nursery.spawn(aunprotected)

    @_core.enable_ki_protection
    def gen_protected():
        assert _core.currently_ki_protected()
        yield

    for _ in gen_protected():
        pass

    @_core.disable_ki_protection
    def gen_unprotected():
        assert not _core.currently_ki_protected()
        yield

    for _ in gen_unprotected():
        pass


# This used to be broken due to
#
#   https://bugs.python.org/issue29590
#
# Specifically, after a coroutine is resumed with .throw(), then the stack
# makes it look like the immediate caller is the function that called
# .throw(), not the actual caller. So child() here would have a caller deep in
# the guts of the run loop, and always be protected, even when it shouldn't
# have been. (Solution: we don't use .throw() anymore.)
async def test_ki_enabled_after_yield_briefly():
    @_core.enable_ki_protection
    async def protected():
        await child(True)

    @_core.disable_ki_protection
    async def unprotected():
        await child(False)

    async def child(expected):
        import traceback
        traceback.print_stack()
        assert _core.currently_ki_protected() == expected
        await _core.yield_briefly()
        traceback.print_stack()
        assert _core.currently_ki_protected() == expected

    await protected()
    await unprotected()


# This also used to be broken due to
#   https://bugs.python.org/issue29590
async def test_generator_based_context_manager_throw():
    @contextlib.contextmanager
    @_core.enable_ki_protection
    def protected_manager():
        assert _core.currently_ki_protected()
        try:
            yield
        finally:
            assert _core.currently_ki_protected()

    with protected_manager():
        assert not _core.currently_ki_protected()

    with pytest.raises(KeyError):
        # This is the one that used to fail
        with protected_manager():
            raise KeyError


async def test_agen_protection():
    @_core.enable_ki_protection
    @async_generator
    async def agen_protected1():
        assert _core.currently_ki_protected()
        try:
            await yield_()
        finally:
            assert _core.currently_ki_protected()

    @_core.disable_ki_protection
    @async_generator
    async def agen_unprotected1():
        assert not _core.currently_ki_protected()
        try:
            await yield_()
        finally:
            assert not _core.currently_ki_protected()

    # Swap the order of the decorators:
    @async_generator
    @_core.enable_ki_protection
    async def agen_protected2():
        assert _core.currently_ki_protected()
        try:
            await yield_()
        finally:
            assert _core.currently_ki_protected()

    @async_generator
    @_core.disable_ki_protection
    async def agen_unprotected2():
        assert not _core.currently_ki_protected()
        try:
            await yield_()
        finally:
            assert not _core.currently_ki_protected()

    for agen_fn in [
            agen_protected1, agen_protected2,
            agen_unprotected1, agen_unprotected2,
    ]:
        async for _ in agen_fn():
            assert not _core.currently_ki_protected()

        async with acontextmanager(agen_fn)():
            assert not _core.currently_ki_protected()

        # Another case that's tricky due to:
        #   https://bugs.python.org/issue29590
        with pytest.raises(KeyError):
            async with acontextmanager(agen_fn)():
                raise KeyError


# Test the case where there's no magic local anywhere in the call stack
def test_ki_enabled_out_of_context():
    assert not _core.currently_ki_protected()


def test_ki_protection_works():
    async def sleeper(name, record):
        try:
            while True:
                await _core.yield_briefly()
        except _core.Cancelled:
            record.add((name + " ok"))

    async def raiser(name, record):
        try:
            # os.kill runs signal handlers before returning, so we don't need
            # to worry that the handler will be delayed
            print("killing, protection =", _core.currently_ki_protected())
            ki_self()
        except KeyboardInterrupt:
            print("raised!")
            # Make sure we aren't getting cancelled as well as siginted
            await _core.yield_briefly()
            record.add((name + " raise ok"))
            raise
        else:
            print("didn't raise!")
            # If we didn't raise (b/c protected), then we *should* get
            # cancelled at the next opportunity
            try:
                await _core.yield_indefinitely(lambda _: _core.Abort.SUCCEEDED)
            except _core.Cancelled:
                record.add((name + " cancel ok"))

    # simulated control-C during raiser, which is *unprotected*
    print("check 1")
    record = set()
    async def check_unprotected_kill():
        async with _core.open_nursery() as nursery:
            nursery.spawn(sleeper, "s1", record)
            nursery.spawn(sleeper, "s2", record)
            nursery.spawn(raiser, "r1", record)
    with pytest.raises(KeyboardInterrupt):
        _core.run(check_unprotected_kill)
    assert record == {"s1 ok", "s2 ok", "r1 raise ok"}

    # simulated control-C during raiser, which is *protected*, so the KI gets
    # delivered to the main task instead
    print("check 2")
    record = set()
    async def check_protected_kill():
        async with _core.open_nursery() as nursery:
            nursery.spawn(sleeper, "s1", record)
            nursery.spawn(sleeper, "s2", record)
            nursery.spawn(_core.enable_ki_protection(raiser), "r1", record)
            # __aexit__ blocks, and then receives the KI
    with pytest.raises(KeyboardInterrupt):
        _core.run(check_protected_kill)
    assert record == {"s1 ok", "s2 ok", "r1 cancel ok"}

    # kill at last moment still raises (call_soon until it raises an error,
    # then kill)
    print("check 3")
    async def check_kill_during_shutdown():
        call_soon = _core.current_call_soon_thread_and_signal_safe()
        def kill_during_shutdown():
            assert _core.currently_ki_protected()
            try:
                call_soon(kill_during_shutdown)
            except _core.RunFinishedError:
                # it's too late for regular handling! handle this!
                print("kill! kill!")
                ki_self()
        call_soon(kill_during_shutdown)

    with pytest.raises(KeyboardInterrupt):
        _core.run(check_kill_during_shutdown)

    # KI arrives very early, before main is even spawned
    print("check 4")
    class InstrumentOfDeath:
        def before_run(self):
            ki_self()
    async def main():
        await _core.yield_briefly()
    with pytest.raises(KeyboardInterrupt):
        _core.run(main, instruments=[InstrumentOfDeath()])

    # yield_if_cancelled notices pending KI
    print("check 5")
    @_core.enable_ki_protection
    async def main():
        assert _core.currently_ki_protected()
        ki_self()
        with pytest.raises(KeyboardInterrupt):
            await _core.yield_if_cancelled()
    _core.run(main)

    # KI arrives while main task is not abortable, b/c already scheduled
    print("check 6")
    @_core.enable_ki_protection
    async def main():
        assert _core.currently_ki_protected()
        ki_self()
        await _core.yield_briefly_no_cancel()
        await _core.yield_briefly_no_cancel()
        await _core.yield_briefly_no_cancel()
        with pytest.raises(KeyboardInterrupt):
            await _core.yield_briefly()
    _core.run(main)

    # KI arrives while main task is not abortable, b/c refuses to be aborted
    print("check 7")
    @_core.enable_ki_protection
    async def main():
        assert _core.currently_ki_protected()
        ki_self()
        task = _core.current_task()
        def abort(_):
            _core.reschedule(task, _core.Value(1))
            return _core.Abort.FAILED
        assert await _core.yield_indefinitely(abort) == 1
        with pytest.raises(KeyboardInterrupt):
            await _core.yield_briefly()
    _core.run(main)

    # KI delivered via slow abort
    print("check 8")
    @_core.enable_ki_protection
    async def main():
        assert _core.currently_ki_protected()
        ki_self()
        task = _core.current_task()
        def abort(raise_cancel):
            result = _core.Result.capture(raise_cancel)
            _core.reschedule(task, result)
            return _core.Abort.FAILED
        with pytest.raises(KeyboardInterrupt):
            assert await _core.yield_indefinitely(abort)
        await _core.yield_briefly()
    _core.run(main)

    # KI arrives just before main task exits, so the call_soon machinery is
    # still functioning and will accept the callback to deliver the KI, but by
    # the time the callback is actually run, main has exited and can't be
    # aborted.
    print("check 9")
    @_core.enable_ki_protection
    async def main():
        ki_self()
    with pytest.raises(KeyboardInterrupt):
        _core.run(main)


def test_ki_is_good_neighbor():
    # in the unlikely event someone overwrites our signal handler, we leave
    # the overwritten one be
    try:
        orig = signal.getsignal(signal.SIGINT)

        def my_handler(signum, frame):  # pragma: no cover
            pass
        async def main():
            signal.signal(signal.SIGINT, my_handler)
        _core.run(main)

        assert signal.getsignal(signal.SIGINT) is my_handler
    finally:
        signal.signal(signal.SIGINT, orig)



# Use SSLObject to make a generic wrapper around Stream

# SSL shutdown:
# - call unwrap() on the SSLSocket/SSLObject
# - this sends the "all done here" SSL message
# - but in many practical applications this is neither sent nor checked for,
#   e.g. HTTPS usually ignores it:
#   https://security.stackexchange.com/questions/82028/ssl-tls-is-a-server-always-required-to-respond-to-a-close-notify
#   BUT it is important in some cases, so should be possible to handle
#   properly.
#
# I think the answer is: close is synchronous, and the TLS Stream also has an
# async def unwrap() which sends the close_notify message.
# Possibly we should also default suppress_ragged_eofs to False, unlike the
# stdlib? not sure.

# XX how closely should we match the stdlib API?
# - maybe suppress_ragged_eofs=False is a better default?
# - maybe check crypto folks for advice?
# - this is also interesting: https://bugs.python.org/issue8108#msg102867

# Definitely keep an eye on Cory's TLS API ideas on security-sig etc.

# from ._stream import Stream

# class SSLStream(Stream):
#     async def unwrap(self):
#         # does a clean shutdown (!) by calling SSLObject.unwrap(), sends the
#         # resulting close_notify data, and *then* returns the underlying
#         # stream.
#         XX

#     # XX



# Need to be defined early so they can be imported:
def _public(fn):
    # Used to mark methods on _Runner and on IOManager implementations that
    # should be wrapped as global context-sensitive functions (see the bottom
    # of _run.py for the wrapper implementation).
    fn._public = True
    return fn

def _hazmat(fn):
    # Everything exported by this module gets re-exported as either part of
    # the trio.* namespace or else the trio.hazmat.* namespace. By default,
    # thing go into trio.*. But functions marked with this decorator go into
    # trio.hazmat.* instead. See trio/__init__.py for details.
    fn._hazmat = True
    return fn

__all__ = []

from ._exceptions import *
__all__ += _exceptions.__all__

from ._multierror import *
__all__ += _multierror.__all__

from ._result import *
__all__ += _result.__all__

from ._traps import *
__all__ += _traps.__all__

from ._ki import *
__all__ += _ki.__all__

from ._run import *
__all__ += _run.__all__

from ._parking_lot import *
__all__ += _parking_lot.__all__

from ._unbounded_queue import *
__all__ += _unbounded_queue.__all__


if hasattr(_run, "wait_readable"):
    import socket as _stdlib_socket

    @_hazmat
    async def wait_socket_readable(sock):
        if type(sock) != _stdlib_socket.socket:
            raise TypeError("need a socket")
        await wait_readable(sock)

    @_hazmat
    async def wait_socket_writable(sock):
        if type(sock) != _stdlib_socket.socket:
            raise TypeError("need a socket")
        await wait_writable(sock)

    __all__ += ["wait_socket_readable", "wait_socket_writable"]



import operator
from collections import deque

import attr

from . import _core
from ._util import aiter_compat

__all__ = ["Event", "Semaphore", "Lock", "Condition", "Queue"]

@attr.s(slots=True, repr=False, cmp=False, hash=False)
class Event:
    """A waitable boolean value useful for inter-task synchronization,
    inspired by :class:`threading.Event`.

    An event object manages an internal boolean flag, which is initially
    False.

    """

    _lot = attr.ib(default=attr.Factory(_core.ParkingLot), init=False)
    _flag = attr.ib(default=False, init=False)

    def is_set(self):
        """Return the current value of the internal flag.

        """
        return self._flag

    @_core.enable_ki_protection
    def set(self):
        """Set the internal flag value to True, and wake any waiting tasks.

        """
        self._flag = True
        self._lot.unpark_all()

    def clear(self):
        """Set the internal flag value to False.

        """
        self._flag = False

    async def wait(self):
        """Block until the internal flag value becomes True.

        If it's already True, then this method is still a yield point, but
        otherwise returns immediately.

        """
        if self._flag:
            await _core.yield_briefly()
        else:
            await self._lot.park()

    def statistics(self):
        """Return an object containing debugging information.

        Currently the following fields are defined:

        * ``tasks_waiting``: The number of tasks blocked on this event's
          :meth:`wait` method.

        """
        return self._lot.statistics()


def async_cm(cls):
    @_core.enable_ki_protection
    async def __aenter__(self):
        await self.acquire()
    __aenter__.__qualname__ = cls.__qualname__ + ".__aenter__"
    cls.__aenter__ = __aenter__

    @_core.enable_ki_protection
    async def __aexit__(self, *args):
        self.release()
    __aexit__.__qualname__ = cls.__qualname__ + ".__aexit__"
    cls.__aexit__ = __aexit__
    return cls


@async_cm
class Semaphore:
    """A `semaphore <https://en.wikipedia.org/wiki/Semaphore_(programming)>`__.

    A semaphore holds an integer value, which can be incremented by
    calling :meth:`release` and decremented by calling :meth:`acquire` – but
    the value is never allowed to drop below zero. If the value is zero, then
    :meth:`acquire` will block until someone calls :meth:`release`.

    This is a very flexible synchronization object, but perhaps the most
    common use is to represent a resource with some bounded supply. For
    example, if you want to make sure that there are never more than four
    tasks simultaneously performing some operation, you could do something
    like::

       # Allocate a shared Semaphore object, and somehow distribute it to all
       # your tasks. NB: max_value=4 isn't technically necessary, but can
       # help catch errors.
       sem = trio.Semaphore(4, max_value=4)

       # Then when you perform the operation:
       async with sem:
           await perform_operation()

    This object's interface is similar to, but different from, that of
    :class:`threading.Semaphore`.

    A :class:`Semaphore` object can be used as an async context manager; it
    blocks on entry but not on exit.

    Args:
      initial_value (int): A non-negative integer giving semaphore's initial
        value.
      max_value (int or None): If given, makes this a "bounded" semaphore that
        raises an error if the value is about to exceed the given
        ``max_value``.

    """
    def __init__(self, initial_value, *, max_value=None):
        if not isinstance(initial_value, int):
            raise TypeError("initial_value must be an int")
        if initial_value < 0:
            raise ValueError("initial value must be >= 0")
        if max_value is not None:
            if not isinstance(max_value, int):
                raise TypeError("max_value must be None or an int")
            if max_value < initial_value:
                raise ValueError("max_values must be >= initial_value")

        # Invariants:
        # bool(self._lot) implies self._value == 0
        # (or equivalently: self._value > 0 implies not self._lot)
        self._lot = _core.ParkingLot()
        self._value = initial_value
        self._max_value = max_value

    def __repr__(self):
        if self._max_value is None:
            max_value_str = ""
        else:
            max_value_str = ", max_value={}".format(self._max_value)
        return ("<trio.Semaphore({}{}) at {:#x}>"
                .format(self._value, max_value_str, id(self)))

    @property
    def value(self):
        """The current value of the semaphore.

        """
        return self._value

    @property
    def max_value(self):
        """The maximum allowed value. May be None to indicate no limit.

        """
        return self._max_value

    @_core.enable_ki_protection
    def acquire_nowait(self):
        """Attempt to decrement the semaphore value, without blocking.

        Raises:
          WouldBlock: if the value is zero.

        """
        if self._value > 0:
            assert not self._lot
            self._value -= 1
        else:
            raise _core.WouldBlock

    @_core.enable_ki_protection
    async def acquire(self):
        """Decrement the semaphore value, blocking if necessary to avoid
        letting it drop below zero.

        """
        await _core.yield_if_cancelled()
        try:
            self.acquire_nowait()
        except _core.WouldBlock:
            await self._lot.park()
        else:
            await _core.yield_briefly_no_cancel()

    @_core.enable_ki_protection
    def release(self):
        """Increment the semaphore value, possibly waking a task blocked in
        :meth:`acquire`.

        Raises:
          ValueError: if incrementing the value would cause it to exceed
              :attr:`max_value`.

        """
        if self._lot:
            assert self._value == 0
            self._lot.unpark(count=1)
        else:
            if self._max_value is not None and self._value == self._max_value:
                raise ValueError("semaphore released too many times")
            self._value += 1

    def statistics(self):
        """Return an object containing debugging information.

        Currently the following fields are defined:

        * ``tasks_waiting``: The number of tasks blocked on this semaphore's
          :meth:`acquire` method.

        """
        return self._lot.statistics()


@attr.s(frozen=True)
class _LockStatistics:
    locked = attr.ib()
    owner = attr.ib()
    tasks_waiting = attr.ib()

@async_cm
@attr.s(slots=True, cmp=False, hash=False, repr=False)
class Lock:
    """A classic `mutex
    <https://en.wikipedia.org/wiki/Lock_(computer_science)>`__.

    This is a non-reentrant, single-owner lock. Unlike
    :class:`threading.Lock`, only the owner of the lock is allowed to release
    it.

    A :class:`Lock` object can be used as an async context manager; it
    blocks on entry but not on exit.

    """

    _lot = attr.ib(default=attr.Factory(_core.ParkingLot), init=False)
    _owner = attr.ib(default=None, init=False)

    def __repr__(self):
        if self.locked():
            s1 = "locked"
            s2 = " with {} waiters".format(len(self._lot))
        else:
            s1 = "unlocked"
            s2 = ""
        return "<{} trio.Lock object at {:#x}{}>".format(s1, id(self), s2)

    def locked(self):
        """Check whether the lock is currently held.

        Returns:
          bool: True if the lock is held, False otherwise.

        """
        return self._owner is not None

    @_core.enable_ki_protection
    def acquire_nowait(self):
        """Attempt to acquire the lock, without blocking.

        Raises:
          WouldBlock: if the lock is held.

        """

        task = _core.current_task()
        if self._owner is task:
            raise RuntimeError("attempt to re-acquire an already held Lock")
        elif self._owner is None and not self._lot:
            # No-one owns it
            self._owner = task
        else:
            raise _core.WouldBlock

    @_core.enable_ki_protection
    async def acquire(self):
        """Acquire the lock, blocking if necessary.

        """
        await _core.yield_if_cancelled()
        try:
            self.acquire_nowait()
        except _core.WouldBlock:
            # NOTE: it's important that the contended acquire path is just
            # "_lot.park()", because that's how Condition.wait() acquires the
            # lock as well.
            await self._lot.park()
        else:
            await _core.yield_briefly_no_cancel()

    @_core.enable_ki_protection
    def release(self):
        """Release the lock.

        Raises:
          RuntimeError: if the calling task does not hold the lock.

        """
        task = _core.current_task()
        if task is not self._owner:
            raise RuntimeError("can't release a Lock you don't own")
        if self._lot:
            (self._owner,) = self._lot.unpark(count=1)
        else:
            self._owner = None

    def statistics(self):
        """Return an object containing debugging information.

        Currently the following fields are defined:

        * ``locked``: boolean indicating whether the lock is held.
        * ``owner``: the :class:`Task` currently holding the lock, or None if
          the lock is not held.
        * ``tasks_waiting``: The number of tasks blocked on this lock's
          :meth:`acquire` method.

        """
        return _LockStatistics(
            locked=self.locked(),
            owner=self._owner,
            tasks_waiting=len(self._lot),
        )


@attr.s(frozen=True)
class _ConditionStatistics:
    tasks_waiting = attr.ib()
    lock_statistics = attr.ib()

@async_cm
class Condition:
    """A classic `condition variable
    <https://en.wikipedia.org/wiki/Monitor_(synchronization)>`__, similar to
    :class:`threading.Condition`.

    A :class:`Condition` object can be used as an async context manager to
    acquire the underlying lock; it blocks on entry but not on exit.

    Args:
      lock (Lock): the lock object to use. If given, must be a
          :class:`trio.Lock`. If None, a new :class:`Lock` will be allocated
          and used.

    """
    def __init__(self, lock=None):
        if lock is None:
            lock = Lock()
        if not isinstance(lock, Lock):
            raise TypeError("lock must be a trio.Lock")
        self._lock = lock
        self._lot = _core.ParkingLot()

    def locked(self):
        """Check whether the underlying lock is currently held.

        Returns:
          bool: True if the lock is held, False otherwise.

        """
        return self._lock.locked()

    def acquire_nowait(self):
        """Attempt to acquire the underlying lock, without blocking.

        Raises:
          WouldBlock: if the lock is currently held.

        """
        return self._lock.acquire_nowait()

    async def acquire(self):
        """Acquire the underlying lock, blocking if necessary.

        """
        await self._lock.acquire()

    def release(self):
        """Release the underlying lock.

        """
        self._lock.release()

    @_core.enable_ki_protection
    async def wait(self):
        """Wait for another thread to call :meth:`notify` or
        :meth:`notify_all`.

        When calling this method, you must hold the lock. It releases the lock
        while waiting, and then re-acquires it before waking up.

        There is a subtlety with how this method interacts with cancellation:
        when cancelled it will block to re-acquire the lock before raising
        :exc:`Cancelled`. This may cause cancellation to be less prompt than
        expected. The advantage is that it makes code like this work::

           async with condition:
               await condition.wait()

        If we didn't re-acquire the lock before waking up, and :meth:`wait`
        were cancelled here, then we'd crash in ``condition.__aexit__`` when
        we tried to release the lock we no longer held.

        Raises:
          RuntimeError: if the calling task does not hold the lock.

        """
        if _core.current_task() is not self._lock._owner:
            raise RuntimeError("must hold the lock to wait")
        self.release()
        # NOTE: we go to sleep on self._lot, but we'll wake up on
        # self._lock._lot. That's all that's required to acquire a Lock.
        try:
            await self._lot.park()
        except:
            with _core.open_cancel_scope(shield=True):
                await self.acquire()
            raise

    def notify(self, n=1):
        """Wake one or more tasks that are blocked in :meth:`wait`.

        Args:
          n (int): The number of tasks to wake.

        Raises:
          RuntimeError: if the calling task does not hold the lock.

        """
        if _core.current_task() is not self._lock._owner:
            raise RuntimeError("must hold the lock to notify")
        self._lot.repark(self._lock._lot, count=n)

    def notify_all(self):
        """Wake all tasks that are currently blocked in :meth:`wait`.

        Raises:
          RuntimeError: if the calling task does not hold the lock.

        """
        if _core.current_task() is not self._lock._owner:
            raise RuntimeError("must hold the lock to notify")
        self._lot.repark_all(self._lock._lot)

    def statistics(self):
        """Return an object containing debugging information.

        Currently the following fields are defined:

        * ``tasks_waiting``: The number of tasks blocked on this condition's
          :meth:`wait` method.
        * ``lock_statistics``: The result of calling the underlying
          :class:`Lock`\s  :meth:`~Lock.statistics` method.

        """
        return _ConditionStatistics(
            tasks_waiting=len(self._lot),
            lock_statistics=self._lock.statistics(),
        )


@attr.s(frozen=True)
class _QueueStats:
    qsize = attr.ib()
    capacity = attr.ib()
    tasks_waiting_put = attr.ib()
    tasks_waiting_get = attr.ib()
    tasks_waiting_join = attr.ib()

# Like queue.Queue, with the notable difference that the capacity argument is
# mandatory.
class Queue:
    """A bounded queue suitable for inter-task communication.

    This class is generally modelled after :class:`queue.Queue`, but with the
    major difference that it is always bounded. For an unbounded queue, see
    :class:`trio.UnboundedQueue`.

    A :class:`Queue` object can be used as an asynchronous iterator, that
    dequeues objects one at a time. I.e., these two loops are equivalent::

       async for obj in queue:
           ...

       while True:
           obj = await queue.get()
           ...

    Args:
      capacity (int): The maximum number of items allowed in the queue before
          :meth:`put` blocks. Choosing a sensible value here is important to
          ensure that backpressure is communicated promptly and avoid
          unnecessary latency. If in doubt, use 1.

    """

    def __init__(self, capacity):
        if not isinstance(capacity, int):
            raise TypeError("capacity must be an integer")
        if capacity < 1:
            raise ValueError("capacity must be >= 1")
        # Invariants:
        #   get_semaphore.value() == len(self._data)
        #   put_semaphore.value() + get_semaphore.value() = capacity
        self.capacity = operator.index(capacity)
        self._put_semaphore = Semaphore(capacity, max_value=capacity)
        self._get_semaphore = Semaphore(0, max_value=capacity)
        self._data = deque()
        self._join_lot = _core.ParkingLot()
        self._unprocessed = 0

    def __repr__(self):
        return ("<Queue({}) at {:#x} holding {} items>"
                .format(self.capacity, id(self), len(self._data)))

    def qsize(self):
        """Returns the number of items currently in the queue.

        There is some subtlety to interpreting this method's return value: see
        `issue #63 <https://github.com/python-trio/trio/issues/63>`__.

        """
        return len(self._data)

    def full(self):
        """Returns True if the queue is at capacity, False otherwise.

        There is some subtlety to interpreting this method's return value: see
        `issue #63 <https://github.com/python-trio/trio/issues/63>`__.

        """
        return len(self._data) == self.capacity

    def empty(self):
        """Returns True if the queue is empty, False otherwise.

        There is some subtlety to interpreting this method's return value: see
        `issue #63 <https://github.com/python-trio/trio/issues/63>`__.

        """
        return not self._data

    def _put_protected(self, obj):
        self._data.append(obj)
        self._unprocessed += 1
        self._get_semaphore.release()

    @_core.enable_ki_protection
    def put_nowait(self, obj):
        """Attempt to put an object into the queue, without blocking.

        Args:
          obj (object): The object to enqueue.

        Raises:
          WouldBlock: if the queue is full.

        """
        self._put_semaphore.acquire_nowait()
        self._put_protected(obj)

    @_core.enable_ki_protection
    async def put(self, obj):
        """Put an object into the queue, blocking if necessary.

        Args:
          obj (object): The object to enqueue.

        """
        await self._put_semaphore.acquire()
        self._put_protected(obj)

    def _get_protected(self):
        self._put_semaphore.release()
        return self._data.popleft()

    @_core.enable_ki_protection
    def get_nowait(self):
        """Attempt to get an object from the queue, without blocking.

        Returns:
          object: The dequeued object.

        Raises:
          WouldBlock: if the queue is empty.

        """
        self._get_semaphore.acquire_nowait()
        return self._get_protected()

    @_core.enable_ki_protection
    async def get(self):
        """Get an object from the queue, blocking is necessary.

        Returns:
          object: The dequeued object.

        """
        await self._get_semaphore.acquire()
        return self._get_protected()

    @_core.enable_ki_protection
    def task_done(self):
        """Decrement the count of unfinished work.

        Each :class:`Queue` object keeps a count of unfinished work, which
        starts at zero and is incremented after each successful
        :meth:`put`. This method decrements it again. When the count reaches
        zero, any tasks blocked in :meth:`join` are woken.

        """
        self._unprocessed -= 1
        if self._unprocessed == 0:
            self._join_lot.unpark_all()

    async def join(self):
        """Block until the count of unfinished work reaches zero.

        See :meth:`task_done` for details.

        """
        if self._unprocessed == 0:
            await _core.yield_briefly()
        else:
            await self._join_lot.park()

    @aiter_compat
    def __aiter__(self):
        return self

    async def __anext__(self):
        return await self.get()

    def statistics(self):
        """Returns an object containing debugging information.

        Currently the following fields are defined:

        * ``qsize``: The number of items currently in the queue.
        * ``capacity``: The maximum number of items the queue can hold.
        * ``tasks_waiting_put``: The number of tasks blocked on this queue's
          :meth:`put` method.
        * ``tasks_waiting_get``: The number of tasks blocked on this queue's
          :meth:`get` method.
        * ``tasks_waiting_join``: The number of tasks blocked on this queue's
          :meth:`join` method.

        """
        return _QueueStats(
            qsize=len(self._data),
            capacity=self.capacity,
            tasks_waiting_put=self._put_semaphore.statistics().tasks_waiting,
            tasks_waiting_get=self._get_semaphore.statistics().tasks_waiting,
            tasks_waiting_join=self._join_lot.statistics().tasks_waiting)



# Utilities for testing

import pytest

# See trio/tests/conftest.py for the other half of this
slow = pytest.mark.skipif(
    not pytest.config.getoption("--run-slow"),
    reason="use --run-slow to run slow tests",
)

from ... import _core

# template is like:
#   [1, {2.1, 2.2}, 3] -> matches [1, 2.1, 3] or [1, 2.2, 3]
def check_sequence_matches(seq, template):
    i = 0
    for pattern in template:
        if not isinstance(pattern, set):
            pattern = {pattern}
        got = set(seq[i:i + len(pattern)])
        assert got == pattern
        i += len(got)



import pytest

import os
import socket as stdlib_socket
import inspect

from .. import _core
from .. import socket as tsocket
from ..testing import assert_yields, wait_all_tasks_blocked

################################################################
# utils
################################################################

class MonkeypatchedGAI:
    def __init__(self, orig_getaddrinfo):
        self._orig_getaddrinfo = orig_getaddrinfo
        self._responses = {}
        self.record = []

    # get a normalized getaddrinfo argument tuple
    def _frozenbind(self, *args, **kwargs):
        sig = inspect.signature(self._orig_getaddrinfo)
        bound = sig.bind(*args, **kwargs)
        bound.apply_defaults()
        frozenbound = bound.args
        assert not bound.kwargs
        return frozenbound

    def set(self, response, *args, **kwargs):
        self._responses[self._frozenbind(*args, **kwargs)] = response

    def getaddrinfo(self, *args, **kwargs):
        bound = self._frozenbind(*args, **kwargs)
        self.record.append(bound)
        if bound in self._responses:
            return self._responses[bound]
        elif bound[-1] & stdlib_socket.AI_NUMERICHOST:
            return self._orig_getaddrinfo(*args, **kwargs)
        else:
            raise RuntimeError(
                "gai called with unexpected arguments {}".format(bound))


@pytest.fixture
def monkeygai(monkeypatch):
    controller = MonkeypatchedGAI(stdlib_socket.getaddrinfo)
    monkeypatch.setattr(stdlib_socket, "getaddrinfo", controller.getaddrinfo)
    return controller


async def test__try_sync():
    with assert_yields():
        async with tsocket._try_sync():
            pass

    with assert_yields():
        with pytest.raises(KeyError):
            async with tsocket._try_sync():
                raise KeyError

    async with tsocket._try_sync():
        raise BlockingIOError

    def _is_ValueError(exc):
        return isinstance(exc, ValueError)

    async with tsocket._try_sync(_is_ValueError):
        raise ValueError

    with assert_yields():
        with pytest.raises(BlockingIOError):
            async with tsocket._try_sync(_is_ValueError):
                raise BlockingIOError


################################################################
# basic re-exports
################################################################

def test_socket_has_some_reexports():
    assert tsocket.SOL_SOCKET == stdlib_socket.SOL_SOCKET
    assert tsocket.TCP_NODELAY == stdlib_socket.TCP_NODELAY
    assert tsocket.gaierror == stdlib_socket.gaierror
    assert tsocket.ntohs == stdlib_socket.ntohs


################################################################
# name resolution
################################################################

async def test_getaddrinfo(monkeygai):
    # Simple non-blocking non-error cases, ipv4 and ipv6:
    with assert_yields():
        res = await tsocket.getaddrinfo(
            "127.0.0.1", "12345", type=tsocket.SOCK_STREAM)
    def check(got, expected):
        # win32 returns 0 for the proto field
        def without_proto(gai_tup):
            return gai_tup[:2] + (0,) + gai_tup[3:]
        expected2 = [without_proto(gt) for gt in expected]
        assert got == expected or got == expected2

    check(res, [
        (tsocket.AF_INET,  # 127.0.0.1 is ipv4
         tsocket.SOCK_STREAM,
         tsocket.IPPROTO_TCP,
         "",
         ("127.0.0.1", 12345)),
    ])

    with assert_yields():
        res = await tsocket.getaddrinfo(
            "::1", "12345", type=tsocket.SOCK_DGRAM)
    check(res, [
        (tsocket.AF_INET6,
         tsocket.SOCK_DGRAM,
         tsocket.IPPROTO_UDP,
         "",
         ("::1", 12345, 0, 0)),
    ])

    monkeygai.set("x", "host", "port", family=0, type=0, proto=0, flags=0)
    with assert_yields():
        res = await tsocket.getaddrinfo("host", "port")
    assert res == "x"
    assert monkeygai.record[-1] == ("host", "port", 0, 0, 0, 0)

    # check raising an error from a non-blocking getaddrinfo
    with assert_yields():
        with pytest.raises(tsocket.gaierror) as excinfo:
            await tsocket.getaddrinfo("::1", "12345", type=-1)
    # Linux, Windows
    expected_errnos = {tsocket.EAI_SOCKTYPE}
    # MacOS
    if hasattr(tsocket, "EAI_BADHINTS"):
        expected_errnos.add(tsocket.EAI_BADHINTS)
    assert excinfo.value.errno in expected_errnos

    # check raising an error from a blocking getaddrinfo (exploits the fact
    # that monkeygai raises if it gets a non-numeric request it hasn't been
    # given an answer for)
    with assert_yields():
        with pytest.raises(RuntimeError):
            await tsocket.getaddrinfo("asdf", "12345")


async def test_getfqdn(monkeypatch):
    def my_getfqdn(name=""):
        return "x{}x".format(name)
    monkeypatch.setattr(stdlib_socket, "getfqdn", my_getfqdn)
    with assert_yields():
        assert await tsocket.getfqdn() == "xx"
    with assert_yields():
        assert await tsocket.getfqdn("foo") == "xfoox"


async def test_getnameinfo():
    ni_numeric = stdlib_socket.NI_NUMERICHOST | stdlib_socket.NI_NUMERICSERV
    with assert_yields():
        assert (await tsocket.getnameinfo(("127.0.0.1", 1234), ni_numeric)
                == ("127.0.0.1", "1234"))
    with assert_yields():
        with pytest.raises(tsocket.gaierror):
            # getnameinfo requires a numeric address as input
            await tsocket.getnameinfo(("google.com", 80), 0)


################################################################
# constructors
################################################################

async def test_from_stdlib_socket():
    sa, sb = stdlib_socket.socketpair()
    with sa, sb:
        ta = tsocket.from_stdlib_socket(sa)
        assert sa.fileno() == ta.fileno()
        await ta.sendall(b"xxx")
        assert sb.recv(3) == b"xxx"

    # rejects other types
    with pytest.raises(TypeError):
        tsocket.from_stdlib_socket(1)
    class MySocket(stdlib_socket.socket):
        pass
    mysock = MySocket()
    with pytest.raises(TypeError):
        tsocket.from_stdlib_socket(mysock)

async def test_from_fd():
    sa, sb = stdlib_socket.socketpair()
    ta = tsocket.fromfd(sa.fileno(), sa.family, sa.type, sa.proto)
    with sa, sb, ta:
        assert ta.fileno() != sa.fileno()
        await ta.sendall(b"xxx")
        assert sb.recv(3) == b"xxx"


async def test_socketpair_simple():
    async def child(sock):
        print("sending hello")
        await sock.sendall(b"hello!")
        buf = bytearray()
        while buf != b"hello!":
            print("reading", buf)
            buf += await sock.recv(10)
        return "ok"

    a, b = tsocket.socketpair()
    with a, b:
        async with _core.open_nursery() as nursery:
            task1 = nursery.spawn(child, a)
            task2 = nursery.spawn(child, b)
    assert task1.result.unwrap() == "ok"
    assert task2.result.unwrap() == "ok"


@pytest.mark.skipif(not hasattr(tsocket, "fromshare"), reason="windows only")
async def test_fromshare():
    a, b = tsocket.socketpair()
    with a, b:
        # share with ourselves
        shared = a.share(os.getpid())
        a2 = tsocket.fromshare(shared)
        with a2:
            assert a.fileno() != a2.fileno()
            await a2.sendall(b"xxx")
            assert await b.recv(3) == b"xxx"


async def test_socket():
    with tsocket.socket() as s:
        assert isinstance(s, tsocket.SocketType)
        assert s.family == tsocket.AF_INET

    with tsocket.socket(tsocket.AF_INET6, tsocket.SOCK_DGRAM) as s:
        assert isinstance(s, tsocket.SocketType)
        assert s.family == tsocket.AF_INET6


################################################################
# SocketType
################################################################

async def test_SocketType_basics():
    sock = tsocket.socket()
    with sock as cm_enter_value:
        assert cm_enter_value is sock
        assert isinstance(sock.fileno(), int)
        assert not sock.get_inheritable()
        sock.set_inheritable(True)
        assert sock.get_inheritable()

        sock.setsockopt(tsocket.IPPROTO_TCP, tsocket.TCP_NODELAY, False)
        assert not sock.getsockopt(tsocket.IPPROTO_TCP, tsocket.TCP_NODELAY)
        sock.setsockopt(tsocket.IPPROTO_TCP, tsocket.TCP_NODELAY, True)
        assert sock.getsockopt(tsocket.IPPROTO_TCP, tsocket.TCP_NODELAY)
    # closed sockets have fileno() == -1
    assert sock.fileno() == -1

    # smoke test
    repr(sock)

    # detach
    with tsocket.socket() as sock:
        fd = sock.fileno()
        assert sock.detach() == fd
        assert sock.fileno() == -1

    # close
    sock = tsocket.socket()
    assert sock.fileno() >= 0
    sock.close()
    assert sock.fileno() == -1

    # share was tested above together with fromshare

    # check __dir__
    assert "family" in dir(sock)
    assert "recv" in dir(sock)
    assert "setsockopt" in dir(sock)

    # our __getattr__ handles unknown names
    with pytest.raises(AttributeError):
        sock.asdf

    # type family proto
    stdlib_sock = stdlib_socket.socket()
    sock = tsocket.from_stdlib_socket(stdlib_sock)
    assert sock.type == stdlib_sock.type
    assert sock.family == stdlib_sock.family
    assert sock.proto == stdlib_sock.proto
    sock.close()


async def test_SocketType_dup():
    a, b = tsocket.socketpair()
    with a, b:
        a2 = a.dup()
        with a2:
            assert isinstance(a2, tsocket.SocketType)
            assert a2.fileno() != a.fileno()
            a.close()
            await a2.sendall(b"xxx")
            assert await b.recv(3) == b"xxx"


async def test_SocketType_shutdown():
    a, b = tsocket.socketpair()
    with a, b:
        await a.sendall(b"xxx")
        assert await b.recv(3) == b"xxx"
        a.shutdown(tsocket.SHUT_WR)
        assert await b.recv(3) == b""
        await b.sendall(b"yyy")
        assert await a.recv(3) == b"yyy"


async def test_SocketType_simple_server():
    # listen, bind, accept, connect, getpeername, getsockname
    with tsocket.socket() as listener, tsocket.socket() as client:
        listener.bind(("127.0.0.1", 0))
        listener.listen(20)
        addr = listener.getsockname()
        async with _core.open_nursery() as nursery:
            nursery.spawn(client.connect, addr)
            accept_task = nursery.spawn(listener.accept)
        server, client_addr = accept_task.result.unwrap()
        with server:
            assert client_addr == server.getpeername() == client.getsockname()
            await server.sendall(b"xxx")
            assert await client.recv(3) == b"xxx"


async def test_SocketType_resolve():
    sock4 = tsocket.socket(family=tsocket.AF_INET)
    with assert_yields():
        assert await sock4.resolve_local_address((None, 80)) == ("0.0.0.0", 80)
    with assert_yields():
        assert (await sock4.resolve_remote_address((None, 80))
                == ("127.0.0.1", 80))

    sock6 = tsocket.socket(family=tsocket.AF_INET6)
    with assert_yields():
        assert (await sock6.resolve_local_address((None, 80))
                == ("::", 80, 0, 0))
    with assert_yields():
        assert (await sock6.resolve_remote_address((None, 80))
                == ("::1", 80, 0, 0))

    # AI_PASSIVE only affects the wildcard address, so for everything else
    # resolve_local_address and resolve_remote_address should work the same:
    for res in ["resolve_local_address", "resolve_remote_address"]:
        async def s4res(*args):
            with assert_yields():
                return await getattr(sock4, res)(*args)
        async def s6res(*args):
            with assert_yields():
                return await getattr(sock6, res)(*args)

        assert await s4res(("1.2.3.4", "http")) == ("1.2.3.4", 80)
        assert await s6res(("1::2", "http")) == ("1::2", 80, 0, 0)

        assert await s6res(("1::2", 80, 1)) == ("1::2", 80, 1, 0)
        assert await s6res(("1::2", 80, 1, 2)) == ("1::2", 80, 1, 2)

        # V4 mapped addresses resolved if V6ONLY if False
        sock6.setsockopt(tsocket.IPPROTO_IPV6, tsocket.IPV6_V6ONLY, False)
        assert await s6res(("1.2.3.4", "http")) == ("::ffff:1.2.3.4", 80, 0, 0)

        # But not if it's true
        sock6.setsockopt(tsocket.IPPROTO_IPV6, tsocket.IPV6_V6ONLY, True)
        with pytest.raises(tsocket.gaierror) as excinfo:
            await s6res(("1.2.3.4", 80))
        # Windows, MacOS
        expected_errnos = {tsocket.EAI_NONAME}
        # Linux
        if hasattr(tsocket, "EAI_ADDRFAMILY"):
            expected_errnos.add(tsocket.EAI_ADDRFAMILY)
        assert excinfo.value.errno in expected_errnos

        # A family where we know nothing about the addresses, so should just
        # pass them through. This should work on Linux, which is enough to
        # smoke test the basic functionality...
        try:
            netlink_sock = tsocket.socket(
                family=tsocket.AF_NETLINK, type=tsocket.SOCK_DGRAM)
        except (AttributeError, OSError):
            pass
        else:
            assert await getattr(netlink_sock, res)("asdf") == "asdf"

        with pytest.raises(ValueError):
            await s4res("1.2.3.4")
        with pytest.raises(ValueError):
            await s4res(("1.2.3.4",))
        with pytest.raises(ValueError):
            await s4res(("1.2.3.4", 80, 0, 0))
        with pytest.raises(ValueError):
            await s6res("1.2.3.4")
        with pytest.raises(ValueError):
            await s6res(("1.2.3.4",))
        with pytest.raises(ValueError):
            await s6res(("1.2.3.4", 80, 0, 0, 0))


async def test_SocketType_requires_preresolved(monkeypatch):
    sock = tsocket.socket()
    with pytest.raises(ValueError):
        sock.bind(("localhost", 0))

    # I don't think it's possible to actually get a gaierror from the way we
    # call getaddrinfo in _check_address, but just in case someone finds a
    # way, check that it propagates correctly
    def gai_oops(*args, **kwargs):
        raise tsocket.gaierror("nope!")
    monkeypatch.setattr(stdlib_socket, "getaddrinfo", gai_oops)
    with pytest.raises(tsocket.gaierror):
        sock.bind(("localhost", 0))


# This tests all the complicated paths through _nonblocking_helper, using recv
# as a stand-in for all the methods that use _nonblocking_helper.
async def test_SocketType_non_blocking_paths():
    a, b = stdlib_socket.socketpair()
    with a, b:
        ta = tsocket.from_stdlib_socket(a)
        b.setblocking(False)

        # cancel before even calling
        b.send(b"1")
        with _core.open_cancel_scope() as cscope:
            cscope.cancel()
            with assert_yields():
                with pytest.raises(_core.Cancelled):
                    await ta.recv(10)
        # immedate success (also checks that the previous attempt didn't
        # actually read anything)
        with assert_yields():
            await ta.recv(10) == b"1"
        # immediate failure
        with assert_yields():
            with pytest.raises(TypeError):
                await ta.recv("haha")
        # block then succeed
        async def do_successful_blocking_recv():
            with assert_yields():
                assert await ta.recv(10) == b"2"
        async with _core.open_nursery() as nursery:
            nursery.spawn(do_successful_blocking_recv)
            await wait_all_tasks_blocked()
            b.send(b"2")
        # block then cancelled
        async def do_cancelled_blocking_recv():
            with assert_yields():
                with pytest.raises(_core.Cancelled):
                    await ta.recv(10)
        async with _core.open_nursery() as nursery:
            nursery.spawn(do_cancelled_blocking_recv)
            await wait_all_tasks_blocked()
            nursery.cancel_scope.cancel()
        # Okay, here's the trickiest one: we want to exercise the path where
        # the task is signaled to wake, goes to recv, but then the recv fails,
        # so it has to go back to sleep and try again. Strategy: have two
        # tasks waiting on two sockets (to work around the rule against having
        # two tasks waiting on the same socket), wake them both up at the same
        # time, and whichever one runs first "steals" the data from the
        # other:
        tb = tsocket.from_stdlib_socket(b)
        async def t1():
            with assert_yields():
                assert await ta.recv(1) == b"a"
            with assert_yields():
                assert await tb.recv(1) == b"b"
        async def t2():
            with assert_yields():
                assert await tb.recv(1) == b"b"
            with assert_yields():
                assert await ta.recv(1) == b"a"
        async with _core.open_nursery() as nursery:
            nursery.spawn(t1)
            nursery.spawn(t2)
            await wait_all_tasks_blocked()
            a.send(b"b")
            b.send(b"a")
            await wait_all_tasks_blocked()
            a.send(b"b")
            b.send(b"a")


# This tests the complicated paths through connect
async def test_SocketType_connect_paths():
    with tsocket.socket() as sock:
        with assert_yields():
            with pytest.raises(ValueError):
                # Should be a tuple
                await sock.connect("localhost")

    # cancelled before we start
    with tsocket.socket() as sock:
        with assert_yields():
            with _core.open_cancel_scope() as cancel_scope:
                cancel_scope.cancel()
                with pytest.raises(_core.Cancelled):
                    await sock.connect(("127.0.0.1", 80))

    # Handling InterruptedError
    class InterruptySocket(stdlib_socket.socket):
        def connect(self, *args, **kwargs):
            if not hasattr(self, "_connect_count"):
                self._connect_count = 0
            self._connect_count += 1
            if self._connect_count < 3:
                raise InterruptedError
            else:
                return super().connect(*args, **kwargs)
    with tsocket.socket() as sock, tsocket.socket() as listener:
        listener.bind(("127.0.0.1", 0))
        listener.listen()
        # Swap in our weird subclass under the trio.socket.SocketType's nose
        sock._sock.close()
        sock._sock = InterruptySocket()
        with assert_yields():
            await sock.connect(listener.getsockname())
        assert sock.getpeername() == listener.getsockname()

    # Cancelled in between the connect() call and the connect completing
    with _core.open_cancel_scope() as cancel_scope:
        with tsocket.socket() as sock, tsocket.socket() as listener:
            listener.bind(("127.0.0.1", 0))
            listener.listen()
            # Swap in our weird subclass under the trio.socket.SocketType's
            # nose -- and then swap it back out again before we hit
            # wait_socket_writable, which insists on a real socket.
            class CancelSocket(stdlib_socket.socket):
                def connect(self, *args, **kwargs):
                    cancel_scope.cancel()
                    sock._sock = stdlib_socket.fromfd(
                        self.detach(), self.family, self.type)
                    sock._sock.connect(*args, **kwargs)
                    # If connect *doesn't* raise, then pretend it did
                    raise BlockingIOError  # pragma: no cover
            sock._sock.close()
            sock._sock = CancelSocket()

            with assert_yields():
                with pytest.raises(_core.Cancelled):
                    await sock.connect(listener.getsockname())
            assert sock.fileno() == -1

    # Failed connect (hopefully after raising BlockingIOError)
    with tsocket.socket() as sock, tsocket.socket() as non_listener:
        # Claim an unused port
        non_listener.bind(("127.0.0.1", 0))
        # ...but don't call listen, so we're guaranteed that connect attempts
        # to it will fail.
        with assert_yields():
            with pytest.raises(OSError):
                await sock.connect(non_listener.getsockname())


async def test_send_recv_variants():
    a, b = tsocket.socketpair()
    with a, b:
        # recv, including with flags
        await a.sendall(b"xxx")
        assert await b.recv(10, tsocket.MSG_PEEK) == b"xxx"
        assert await b.recv(10) == b"xxx"

        # recv_into
        await a.sendall(b"xxx")
        buf = bytearray(10)
        await b.recv_into(buf)
        assert buf == b"xxx" + b"\x00" * 7

        if hasattr(a, "sendmsg"):
            assert await a.sendmsg([b"xxx"], []) == 3
            assert await b.recv(10) == b"xxx"

    a = tsocket.socket(type=tsocket.SOCK_DGRAM)
    b = tsocket.socket(type=tsocket.SOCK_DGRAM)
    with a, b:
        a.bind(("127.0.0.1", 0))
        b.bind(("127.0.0.1", 0))
        # recvfrom
        assert await a.sendto(b"xxx", b.getsockname()) == 3
        (data, addr) = await b.recvfrom(10)
        assert data == b"xxx"
        assert addr == a.getsockname()

        # sendto + flags
        #
        # I can't find any flags that send() accepts... on Linux at least
        # passing MSG_MORE to send_some on a connected UDP socket seems to
        # just be ignored.
        #
        # But there's no MSG_MORE on Windows or MacOS. I guess send_some flags
        # are really not very useful, but at least this tests them a bit.
        if hasattr(tsocket, "MSG_MORE"):
            await a.sendto(b"xxx", tsocket.MSG_MORE, b.getsockname())
            await a.sendto(b"yyy", tsocket.MSG_MORE, b.getsockname())
            await a.sendto(b"zzz", b.getsockname())
            (data, addr) = await b.recvfrom(10)
            assert data == b"xxxyyyzzz"
            assert addr == a.getsockname()

        # recvfrom_into
        assert await a.sendto(b"xxx", b.getsockname()) == 3
        buf = bytearray(10)
        (nbytes, addr) = await b.recvfrom_into(buf)
        assert nbytes == 3
        assert buf == b"xxx" + b"\x00" * 7
        assert addr == a.getsockname()

        if hasattr(b, "recvmsg"):
            assert await a.sendto(b"xxx", b.getsockname()) == 3
            (data, ancdata, msg_flags, addr) = await b.recvmsg(10)
            assert data == b"xxx"
            assert ancdata == []
            assert msg_flags == 0
            assert addr == a.getsockname()

        if hasattr(b, "recvmsg_into"):
            assert await a.sendto(b"xyzw", b.getsockname()) == 4
            buf1 = bytearray(2)
            buf2 = bytearray(3)
            ret = await b.recvmsg_into([buf1, buf2])
            (nbytes, ancdata, msg_flags, addr) = ret
            assert nbytes == 4
            assert buf1 == b"xy"
            assert buf2 == b"zw" + b"\x00"
            assert ancdata == []
            assert msg_flags == 0
            assert addr == a.getsockname()

        if hasattr(a, "sendmsg"):
            assert await a.sendmsg([b"x", b"yz"], [], 0, b.getsockname()) == 3
            assert await b.recvfrom(10) == (b"xyz", a.getsockname())

    a = tsocket.socket(type=tsocket.SOCK_DGRAM)
    b = tsocket.socket(type=tsocket.SOCK_DGRAM)
    with a, b:
        b.bind(("127.0.0.1", 0))
        await a.connect(b.getsockname())
        # sendall on a connected udp socket; each call creates a separate
        # datagram
        await a.sendall(b"xxx")
        await a.sendall(b"yyy")
        assert await b.recv(10) == b"xxx"
        assert await b.recv(10) == b"yyy"


async def test_SocketType_sendall():
    BIG = 10000000

    a, b = tsocket.socketpair()
    with a, b:
        # Check a sendall that has to be split into multiple parts (on most
        # platforms... on Windows every send() either succeeds or fails as a
        # whole)
        async with _core.open_nursery() as nursery:
            send_task = nursery.spawn(a.sendall, b"x" * BIG)
            await wait_all_tasks_blocked()
            nbytes = 0
            while nbytes < BIG:
                nbytes += len(await b.recv(BIG))
            assert send_task.result is not None
            assert nbytes == BIG
            with pytest.raises(BlockingIOError):
                b._sock.recv(1)

    a, b = tsocket.socketpair()
    with a, b:
        # Cancel half-way through
        async with _core.open_nursery() as nursery:
            sent_complete = 0
            async def sendall_until_cancelled():
                nonlocal sent_complete
                # Need to loop to make sure that we actually do block on
                # Windows
                while True:
                    await a.sendall(b"x" * BIG)
                    sent_complete += BIG
            send_task = nursery.spawn(sendall_until_cancelled)
            await wait_all_tasks_blocked()
            nursery.cancel_scope.cancel()
        assert type(send_task.result) is _core.Error
        assert isinstance(send_task.result.error, _core.Cancelled)
        sent_partial = send_task.result.error.partial_result.bytes_sent
        a.close()
        sent_total = 0
        while True:
            got = len(await b.recv(BIG))
            if not got:
                break
            sent_total += got
        assert sent_complete + sent_partial == sent_total

    a, b = tsocket.socketpair()
    with a, b:
        # A different error
        a.close()
        with pytest.raises(OSError) as excinfo:
            await a.sendall(b"x")
        assert excinfo.value.partial_result.bytes_sent == 0



from setuptools import setup, find_packages

exec(open("trio/_version.py", encoding="utf-8").read())

LONG_DESC = """\
Trio is an experimental attempt to produce a production-quality,
`permissively licensed
<https://github.com/python-trio/trio/blob/master/LICENSE>`__,
async/await-native I/O library for Python, with an emphasis on
**usability** and **correctness** – we want to make it *easy* to
get things *right*.

Docs: https://trio.readthedocs.io

Issues: https://github.com/python-trio/trio/issues

Repository: https://github.com/python-trio/trio
"""

setup(name="trio",
      version=__version__,
      description="An async/await-native I/O library for humans and snake people",
      long_description=LONG_DESC,
      author="Nathaniel J. Smith",
      author_email="njs@pobox.com",
      url="https://github.com/python-trio/trio",
      license="MIT -or- Apache License 2.0",
      packages=find_packages(),
      install_requires=[
          "attrs",
          "sortedcontainers",
          "async_generator >= 1.6",
          # PEP 508 style, but:
          # https://bitbucket.org/pypa/wheel/issues/181/bdist_wheel-silently-discards-pep-508
          #"cffi; os_name == 'nt'",  # "cffi is required on windows"
      ],
      # Quirky bdist_wheel-specific way:
      # https://wheel.readthedocs.io/en/latest/#defining-conditional-dependencies
      # also supported by pip and setuptools, as long as they're vaguely
      # recent
      extras_require={
          ":os_name == 'nt'": ["cffi"],
      },
      python_requires=">=3.5",
      classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: BSD",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: System :: Networking",
      ],
)
